-- -------------------------------------------------------------------------- --
-- DrainSouler by kunda                                                       --
-- -------------------------------------------------------------------------- --
--                                                                            --
-- DrainSouler is a WoW addon for affliction warlocks.                        --
-- DrainSouler shows all important data you need to know about the following  --
-- channeling spells:                                                         --
--  - Drain Life    (spellID =    689)                                        --
--  - Drain Soul    (spellID =   1120)                                        --
--  - Harvest Life  (spellID = 108371)                                        --
--  - Malefic Grasp (spellID = 103103)                                        --
--                                                                            --
-- Features:                                                                  --
-- # Display:                                                                 --
--   - Frame: Default size: 90x22px                                           --
--   - Frame: Movable & scalable                                              --
--   - Frame: Click through ('Lock DrainSouler')                              --
--   - HP (in percent)                                                        --
--   - SpellIcon (right side)                                                 --
--   - WarningIcon (left side - Drain Soul Icon)                              --
--                 (visible if target HP is at or below 20%)                  --
--   - ChannelTimer (a bar for the whole channeling duration)                 --
--   - TickTimer (a bar that indicates when the next Tick is expected)        --
--   - Tick (the number of damage ticks since caststart)                      --
--   - DMG (damage from last tick)                                            --
--   - Detailed information (DISABLED by DEFAULT)                             --
--      numTicks | tick duration in ms | complete duration in ms              --
--      Haste +x% | Spell Power +x%                                           --
--        (+x% is shown if this stat has changed since caststart)             --
-- # Sound: (DISABLED by DEFAULT)                                             --
--   - Tick Sound (plays a sound when a channeling spell dealt damage)        --
--   - Warning Sound (plays a sound when target HP is at or below 20%)        --
--                -> important for Drain Soul                                 --
-- # Resource friendly:                                                       --
--   - CPU/Memory resources are only used when you login with a WARLOCK       --
--   - CombatLog events are only used when needed (register/unregister)       --
--                                                                            --
-- Configuration:                                                             --
-- # If you log in the first time the 'Configuration' is enabled!             --
-- # 'Esc (Escape) -> Interface -> AddOns -> DrainSouler' to change options!  --
-- # slash commands: '/ds' or '/drainsouler'                                  --
--                                                                            --
-- -------------------------------------------------------------------------- --

-- -------------------------------------------------------------------------- --
--                                    STOP                                    --
--                 You must be a Warlock to enter this code.                  --
--                  If you are not a Warlock please leave.                    --
        if select(2, UnitClass("player")) ~= "WARLOCK" then return end          
-- -------------------------------------------------------------------------- --

-- ---------------------------------------------------------------------------------------------------------------------
DrainSouler_Options = {}                 -- SavedVariable options table
local DrainSouler = CreateFrame("Frame") -- container
local L = DrainSouler_Locales            -- localization table
local GVAR = {}                          -- UI Widgets
local TEMPLATE = {}                      -- Templates

local _G = _G
local GetTime = GetTime
local GetSpellInfo = GetSpellInfo
local GetSpecializationInfoForClassID = GetSpecializationInfoForClassID
local InCombatLockdown = InCombatLockdown
local UnitExists = UnitExists
local UnitGUID = UnitGUID
local UnitHealth = UnitHealth
local UnitHealthMax = UnitHealthMax
local UnitChannelInfo = UnitChannelInfo
local UnitSpellHaste = UnitSpellHaste
local GetSpellBonusDamage = GetSpellBonusDamage
local UnitInVehicle = UnitInVehicle
local UnitHasVehicleUI = UnitHasVehicleUI
local PlaySoundFile = PlaySoundFile
local math_floor = math.floor
local math_min = math.min
local math_max = math.max
local unpack = unpack

local INFO = INFO or "Info"
local CHANNEL = CHANNEL or "Channel"
local STAT_HASTE = STAT_HASTE or "Haste"
local STAT_SPELLPOWER = STAT_SPELLPOWER or "Spell Power"

local mult2 = 10^2
local inCombat
local PlayerGUID

local DrainSoulName    = GetSpellInfo(1120)   --print("DrainSoul", GetSpellInfo(1120))
local DrainLifeName    = GetSpellInfo(689)    --print("DrainLife", GetSpellInfo(689))
local HarvestLifeName  = GetSpellInfo(108371) --print("HarvestLife", GetSpellInfo(108371))
local MaleficGraspName = GetSpellInfo(103103) --print("MaleficGrasp", GetSpellInfo(103103))

if not DrainSoulName or not DrainLifeName or not HarvestLifeName or not MaleficGraspName then
	print("|cffffff7fDrainSouler:|r", "ERROR", "spell unknown")
	print("|cffffff7fDrainSouler:|r", "ERROR", "DrainSoulName:", DrainSoulName, "DrainLifeName:", DrainLifeName, "HarvestLifeName:", HarvestLifeName, "MaleficGraspName:", MaleficGraspName)
	print("|cffffff7fDrainSouler terminated.|r")
	return
end

local channelSpells = { -- baseTickDura = nude and unbuffed
	[DrainSoulName]    = {int = 1, spellID =   1120, spellDura = 12000, baseTickDura = 2000, timerColor = {0.141, 0.509, 0.709}, spellIcon = "Interface\\Icons\\Spell_Shadow_Haunting"},
	[DrainLifeName]    = {int = 2, spellID =    689, spellDura =  6000, baseTickDura = 1000, timerColor = {0.239, 0.725, 0.356}, spellIcon = "Interface\\Icons\\Spell_Shadow_LifeDrain02"},
	[HarvestLifeName]  = {int = 3, spellID = 108371, spellDura =  6000, baseTickDura = 1000, timerColor = {0.239, 0.725, 0.356}, spellIcon = "Interface\\Icons\\Spell_Warlock_HarvestOfLife"},
	[MaleficGraspName] = {int = 4, spellID = 103103, spellDura =  4000, baseTickDura = 1000, timerColor = {0.682, 0.325, 0.792}, spellIcon = "Interface\\Icons\\Ability_Warlock_EverlastingAffliction"},
}

local channelINTspells = {
	[1] = DrainSoulName,
	[2] = DrainLifeName,
	[3] = HarvestLifeName,
	[4] = MaleficGraspName,
}

local configMode

local isAffliction
local currentSpecName
local _, afflictionSpecName = GetSpecializationInfoForClassID(9, 1) -- WARLOCK classID = 9, Affliction Spec = 1

local mobTable = {}
local trashTime = GetTime()
local TickUpdate -- OnUpdate function
local tickDura

local channelTick = 0
local channelDura
local channelStart
local channelRemain
local channeling

local curTickDura
local curTickStart

local startDestGUID
local startHaste
local startDamage

local sizeTimerWidth = 68
local sizeOffset     = 5
local sizeBarHeight  = 12

local function rt(H,E,M,P) return E,P,E,M,H,P,H,M end -- magical 180 degree texture cut center rotation

local Textures = {
	DrainSoulerIcons = {path       = "Interface\\Addons\\DrainSouler\\DrainSouler-texture-icons.tga"}, -- Textures.DrainSoulerIcons.path
	SliderKnob       = {coords     =    {19/64, 30/64,  1/32, 18/32}},
	SliderBG         = {coordsL    =    { 1/64,  6/64, 26/32, 32/32},
	                    coordsM    =    { 9/64, 10/64, 26/32, 32/32},
	                    coordsR    =    {13/64, 18/64, 26/32, 32/32},
	                    coordsLdis =    { 1/64,  6/64,  1/32,  7/32},
	                    coordsMdis =    { 9/64, 10/64,  1/32,  7/32},
	                    coordsRdis =    {13/64, 18/64,  1/32,  7/32}},
	Expand           = {coords     =    { 1/64, 18/64,  8/32, 25/32}},
	Collapse         = {coords     = {rt( 1/64, 18/64,  8/32, 25/32)}}, -- 180 degree rota
}

local soundsTick = {
	[1] = {name = L["No Sound"]},
	[2] = {name = "Beep1 (0.076)",  path = "Interface\\Addons\\DrainSouler\\DrainSouler-sound-beep1.ogg"},
	[3] = {name = "Beep2 (0.043)",  path = "Interface\\Addons\\DrainSouler\\DrainSouler-sound-beep2.ogg"},
	[4] = {name = "Beep3 (0.104)",  path = "Interface\\Addons\\DrainSouler\\DrainSouler-sound-beep3.ogg"},
	[5] = {name = "Deek (0.069)",   path = "Interface\\Addons\\DrainSouler\\DrainSouler-sound-deek.ogg"},
	[6] = {name = "Bump (0.134)",   path = "Interface\\Addons\\DrainSouler\\DrainSouler-sound-bump.ogg"},
	[7] = {name = "Phaser (0.162)", path = "Interface\\Addons\\DrainSouler\\DrainSouler-sound-phaser.ogg"},
	[8] = {name = "Space (0.471)",  path = "Interface\\Addons\\DrainSouler\\DrainSouler-sound-space.ogg"},
}
local soundsWarning = {
	[1]  = {name = L["No Sound"]},
	[2]  = {name = "Alarm1 (0.923)",    path = "Interface\\Addons\\DrainSouler\\DrainSouler-soundwarn-alarm1.ogg"},
	[3]  = {name = "Alarm2 (0.365)",    path = "Interface\\Addons\\DrainSouler\\DrainSouler-soundwarn-alarm2.ogg"},
	[4]  = {name = "Alarm3 (0.940)",    path = "Interface\\Addons\\DrainSouler\\DrainSouler-soundwarn-alarm3.ogg"},
	[5]  = {name = "Bmmmmmm (0.616)",   path = "Interface\\Addons\\DrainSouler\\DrainSouler-soundwarn-bmmmmmm.ogg"},
	[6]  = {name = "Dingding (0.885)",  path = "Interface\\Addons\\DrainSouler\\DrainSouler-soundwarn-dingding.ogg"},
	[7]  = {name = "Electro (0.473)",   path = "Interface\\Addons\\DrainSouler\\DrainSouler-soundwarn-electro.ogg"},
	[8]  = {name = "Horn (0.583)",      path = "Interface\\Addons\\DrainSouler\\DrainSouler-soundwarn-horn.ogg"},
	[9]  = {name = "Radarping (0.819)", path = "Interface\\Addons\\DrainSouler\\DrainSouler-soundwarn-radarping.ogg"}
}
local soundChannel = {
	[1] = {channel = "Master",   name = MASTER_VOLUME},
	[2] = {channel = "SFX",      name = SOUND_VOLUME},
	[3] = {channel = "Music",    name = MUSIC_VOLUME},
	[4] = {channel = "Ambience", name = AMBIENCE_VOLUME}
}

local function NOOP() end

local function Desaturation(texture, desaturation)
	local shaderSupported = texture:SetDesaturated(desaturation)
	if not shaderSupported then
		if desaturation then
			texture:SetVertexColor(0.5, 0.5, 0.5)
		else
			texture:SetVertexColor(1.0, 1.0, 1.0)
		end
	end
end
-- ---------------------------------------------------------------------------------------------------------------------

-- ---------------------------------------------------------------------------------------------------------------------
-- Template BorderTRBL START ----------------------------------------
TEMPLATE.BorderTRBL = function(frame, color) -- TRBL = Top-Right-Bottom-Left
	frame.FrameBorder = frame:CreateTexture(nil, "BORDER")
	frame.FrameBorder:SetPoint("TOPLEFT", 1, -1)
	frame.FrameBorder:SetPoint("BOTTOMRIGHT", -1, 1)
	frame.FrameBorder:SetTexture(0, 0, 0, 1)
	frame.FrameBackground = frame:CreateTexture(nil, "BACKGROUND")
	frame.FrameBackground:SetPoint("TOPLEFT", 0, 0)
	frame.FrameBackground:SetPoint("BOTTOMRIGHT", 0, 0)
	frame.FrameBackground:SetTexture(0.8, 0.2, 0.2, 1)
end
-- Template BorderTRBL END ----------------------------------------

-- Template TextButton START ----------------------------------------
TEMPLATE.DisableTextButton = function(button)
	button.Border:SetTexture(0.4, 0.4, 0.4, 1)
	button:Disable()
end

TEMPLATE.EnableTextButton = function(button, action)
	local buttoncolor
	if action == 1 then
		bordercolor = {0.73, 0.26, 0.21, 1}
	elseif action == 2 then
		bordercolor = {0.43, 0.32, 0.68, 1}
	elseif action == 3 then
		bordercolor = {0.24, 0.46, 0.21, 1}
	else
		bordercolor = {1, 1, 1, 1}
	end
	button.Border:SetTexture(bordercolor[1], bordercolor[2], bordercolor[3], bordercolor[4])
	button:Enable()
end

TEMPLATE.TextButton = function(button, text, action)
	local buttoncolor
	local bordercolor
	if action == 1 then
		button:SetNormalFontObject("GameFontNormal")
		button:SetDisabledFontObject("GameFontDisable") 
		buttoncolor = {0.38, 0, 0, 1}
		bordercolor = {0.73, 0.26, 0.21, 1}
	elseif action == 2 then
		button:SetNormalFontObject("GameFontWhiteSmall")
		button:SetDisabledFontObject("GameFontDisableSmall")
		buttoncolor = {0, 0, 0.5, 1}
		bordercolor = {0.43, 0.32, 0.68, 1}
	elseif action == 3 then
		button:SetNormalFontObject("GameFontWhiteSmall")
		button:SetDisabledFontObject("GameFontDisableSmall")
		buttoncolor = {0, 0.2, 0, 1}
		bordercolor = {0.24, 0.46, 0.21, 1}
	else
		button:SetNormalFontObject("GameFontNormal")
		button:SetDisabledFontObject("GameFontDisable")
		buttoncolor = {0, 0, 0, 1}
		bordercolor = {1, 1, 1, 1}
	end

	button.Background = button:CreateTexture(nil, "BORDER")
	button.Background:SetPoint("TOPLEFT", 1, -1)
	button.Background:SetPoint("BOTTOMRIGHT", -1, 1)
	button.Background:SetTexture(0, 0, 0, 1)

	button.Border = button:CreateTexture(nil, "BACKGROUND")
	button.Border:SetPoint("TOPLEFT", 0, 0)
	button.Border:SetPoint("BOTTOMRIGHT", 0, 0)
	button.Border:SetTexture(bordercolor[1], bordercolor[2], bordercolor[3], bordercolor[4])

	button.Normal = button:CreateTexture(nil, "ARTWORK")
	button.Normal:SetPoint("TOPLEFT", 2, -2)
	button.Normal:SetPoint("BOTTOMRIGHT", -2, 2)
	button.Normal:SetTexture(buttoncolor[1], buttoncolor[2], buttoncolor[3], buttoncolor[4])
	button:SetNormalTexture(button.Normal)

	button.Disabled = button:CreateTexture(nil, "OVERLAY")
	button.Disabled:SetPoint("TOPLEFT", 3, -3)
	button.Disabled:SetPoint("BOTTOMRIGHT", -3, 3)
	button.Disabled:SetTexture(0.6, 0.6, 0.6, 0.2)
	button:SetDisabledTexture(button.Disabled)

	button.Highlight = button:CreateTexture(nil, "OVERLAY")
	button.Highlight:SetPoint("TOPLEFT", 3, -3)
	button.Highlight:SetPoint("BOTTOMRIGHT", -3, 3)
	button.Highlight:SetTexture(0.6, 0.6, 0.6, 0.2)
	button:SetHighlightTexture(button.Highlight)

	button:SetPushedTextOffset(1, -1)	
	button:SetText(text)
end
-- Template TextButton END ----------------------------------------

-- Template CheckButton START ----------------------------------------
TEMPLATE.DisableCheckButton = function(button)
	button.Text:SetTextColor(0.5, 0.5, 0.5)
	button.Border:SetTexture(0.4, 0.4, 0.4, 1)
	button:Disable()
end

TEMPLATE.EnableCheckButton = function(button)
	button.Text:SetTextColor(1, 1, 1)
	button.Border:SetTexture(0.8, 0.2, 0.2, 1)
	button:Enable()
end

TEMPLATE.CheckButton = function(button, size, space, text)
	button.Border = button:CreateTexture(nil, "BACKGROUND")
	button.Border:SetWidth( size )
	button.Border:SetHeight( size )
	button.Border:SetPoint("LEFT", 0, 0)
	button.Border:SetTexture(0.4, 0.4, 0.4, 1)

	button.Background = button:CreateTexture(nil, "BORDER")
	button.Background:SetPoint("TOPLEFT", button.Border, "TOPLEFT", 1, -1)
	button.Background:SetPoint("BOTTOMRIGHT", button.Border, "BOTTOMRIGHT", -1, 1)
	button.Background:SetTexture(0, 0, 0, 1)

	button.Normal = button:CreateTexture(nil, "ARTWORK")
	button.Normal:SetPoint("TOPLEFT", button.Border, "TOPLEFT", 1, -1)
	button.Normal:SetPoint("BOTTOMRIGHT", button.Border, "BOTTOMRIGHT", -1, 1)
	button.Normal:SetTexture(0, 0, 0, 1)
	button:SetNormalTexture(button.Normal)

	button.Push = button:CreateTexture(nil, "ARTWORK")
	button.Push:SetPoint("TOPLEFT", button.Border, "TOPLEFT", 4, -4)
	button.Push:SetPoint("BOTTOMRIGHT", button.Border, "BOTTOMRIGHT", -4, 4)
	button.Push:SetTexture(0.4, 0.4, 0.4, 0.5)
	button:SetPushedTexture(button.Push)

	button.Disabled = button:CreateTexture(nil, "ARTWORK")
	button.Disabled:SetPoint("TOPLEFT", button.Border, "TOPLEFT", 3, -3)
	button.Disabled:SetPoint("BOTTOMRIGHT", button.Border, "BOTTOMRIGHT", -3, 3)
	button.Disabled:SetTexture(0.4, 0.4, 0.4, 0.5)
	button:SetDisabledTexture(button.Disabled)

	button.Checked = button:CreateTexture(nil, "ARTWORK")
	button.Checked:SetWidth( size )
	button.Checked:SetHeight( size )
	button.Checked:SetPoint("LEFT", 0, 0)
	button.Checked:SetTexture("Interface\\Buttons\\UI-CheckBox-Check")
	button:SetCheckedTexture(button.Checked)

	button.Text = button:CreateFontString(nil, "OVERLAY", "GameFontNormalSmall")
	button.Text:SetHeight( size )
	button.Text:SetPoint("LEFT", button.Normal, "RIGHT", space, 0)
	button.Text:SetJustifyH("LEFT")
	button.Text:SetText(text)
	button.Text:SetTextColor(1, 1, 1, 1)

	button:SetWidth(size + space + button.Text:GetStringWidth() + space)
	button:SetHeight(size)

	button.Highlight = button:CreateTexture(nil, "OVERLAY")
	button.Highlight:SetPoint("TOPLEFT", button, "TOPLEFT", 0, 0)
	button.Highlight:SetPoint("BOTTOMRIGHT", button, "BOTTOMRIGHT", 0, 0)
	button.Highlight:SetTexture(1, 1, 1, 0.1)
	button.Highlight:Hide()

	button:SetScript("OnEnter", function() button.Highlight:Show() end)
	button:SetScript("OnLeave", function() button.Highlight:Hide() end)
end
-- Template CheckButton END ----------------------------------------

-- Template PullDownMenu START ----------------------------------------
TEMPLATE.DisablePullDownMenu = function(button)
	button.PullDownMenu:Hide()
	button.PullDownButtonBorder:SetTexture(0.4, 0.4, 0.4, 1)
	button:Disable()
end

TEMPLATE.EnablePullDownMenu = function(button)
	button.PullDownButtonBorder:SetTexture(0.8, 0.2, 0.2, 1)
	button:Enable()
end

TEMPLATE.PullDownMenu = function(button, contentName, buttonText, pulldownWidth, contentNum, func)
	button.PullDownButtonBG = button:CreateTexture(nil, "BORDER")
	button.PullDownButtonBG:SetPoint("TOPLEFT", 1, -1)
	button.PullDownButtonBG:SetPoint("BOTTOMRIGHT", -1, 1)
	button.PullDownButtonBG:SetTexture(0, 0, 0, 1)

	button.PullDownButtonBorder = button:CreateTexture(nil, "BACKGROUND")
	button.PullDownButtonBorder:SetPoint("TOPLEFT", 0, 0)
	button.PullDownButtonBorder:SetPoint("BOTTOMRIGHT", 0, 0)
	button.PullDownButtonBorder:SetTexture(0.4, 0.4, 0.4, 1)

	button.PullDownButtonExpand = button:CreateTexture(nil, "OVERLAY")
	button.PullDownButtonExpand:SetHeight(14)
	button.PullDownButtonExpand:SetWidth(14)
	button.PullDownButtonExpand:SetPoint("RIGHT", button, "RIGHT", -2, 0)
	button.PullDownButtonExpand:SetTexture(Textures.DrainSoulerIcons.path)
	button.PullDownButtonExpand:SetTexCoord(unpack(Textures.Expand.coords))
	button:SetNormalTexture(button.PullDownButtonExpand)

	button.PullDownButtonDisabled = button:CreateTexture(nil, "OVERLAY")
	button.PullDownButtonDisabled:SetPoint("TOPLEFT", 3, -3)
	button.PullDownButtonDisabled:SetPoint("BOTTOMRIGHT", -3, 3)
	button.PullDownButtonDisabled:SetTexture(0.6, 0.6, 0.6, 0.2)
	button:SetDisabledTexture(button.PullDownButtonDisabled)

	button.PullDownButtonHighlight = button:CreateTexture(nil, "OVERLAY")
	button.PullDownButtonHighlight:SetPoint("TOPLEFT", 1, -1)
	button.PullDownButtonHighlight:SetPoint("BOTTOMRIGHT", -1, 1)
	button.PullDownButtonHighlight:SetTexture(0.6, 0.6, 0.6, 0.2)
	button:SetHighlightTexture(button.PullDownButtonHighlight)

	button.PullDownButtonText = button:CreateFontString(nil, "ARTWORK", "GameFontNormalSmall")
	button.PullDownButtonText:SetHeight(sizeBarHeight)
	button.PullDownButtonText:SetPoint("LEFT", sizeOffset+2, 0)
	button.PullDownButtonText:SetJustifyH("LEFT")
	button.PullDownButtonText:SetText(buttonText)
	button.PullDownButtonText:SetTextColor(1, 1, 0.5, 1)

	button.PullDownMenu = CreateFrame("Frame", nil, button)
	TEMPLATE.BorderTRBL(button.PullDownMenu)
	button.PullDownMenu:EnableMouse(true)
	button.PullDownMenu:SetToplevel(true)
	button.PullDownMenu:SetHeight(sizeOffset+(contentNum*sizeBarHeight)+sizeOffset)
	button.PullDownMenu:SetPoint("TOPLEFT", button, "BOTTOMLEFT", 0, 1)
	button.PullDownMenu:Hide()

	local function OnLeave()
		if not button:IsMouseOver() and not button.PullDownMenu:IsMouseOver() then
			button.PullDownMenu:Hide()
			button.PullDownButtonExpand:SetTexCoord(unpack(Textures.Expand.coords))
		end
	end

	local autoWidth = 0
	for i = 1, contentNum do
		if not button.PullDownMenu.Button then button.PullDownMenu.Button = {} end
		button.PullDownMenu.Button[i] = CreateFrame("Button", nil, button.PullDownMenu)
		button.PullDownMenu.Button[i]:SetHeight(sizeBarHeight)
		button.PullDownMenu.Button[i]:SetFrameLevel( button.PullDownMenu:GetFrameLevel() + 5 )
		if i == 1 then
			button.PullDownMenu.Button[i]:SetPoint("TOPLEFT", button.PullDownMenu, "TOPLEFT", sizeOffset, -sizeOffset)
		else
			button.PullDownMenu.Button[i]:SetPoint("TOPLEFT", button.PullDownMenu.Button[(i-1)], "BOTTOMLEFT", 0, 0)
		end

		button.PullDownMenu.Button[i].Text = button.PullDownMenu.Button[i]:CreateFontString(nil, "ARTWORK", "GameFontNormalSmall")
		button.PullDownMenu.Button[i].Text:SetHeight(sizeBarHeight)
		button.PullDownMenu.Button[i].Text:SetPoint("LEFT", 2, 0)
		button.PullDownMenu.Button[i].Text:SetJustifyH("LEFT")
		button.PullDownMenu.Button[i].Text:SetTextColor(1, 1, 1, 1)

		button.PullDownMenu.Button[i]:SetScript("OnLeave", OnLeave)
		button.PullDownMenu.Button[i]:SetScript("OnClick", function()
			button.value1 = button.PullDownMenu.Button[i].value1
			button.PullDownButtonText:SetText( button.PullDownMenu.Button[i].Text:GetText() )
			button.PullDownMenu:Hide()
			button.PullDownButtonExpand:SetTexCoord(unpack(Textures.Expand.coords))
			if func then
				func(button.value1)
			end
		end)

		button.PullDownMenu.Button[i].Highlight = button.PullDownMenu.Button[i]:CreateTexture(nil, "ARTWORK")
		button.PullDownMenu.Button[i].Highlight:SetPoint("TOPLEFT", 0, 0)
		button.PullDownMenu.Button[i].Highlight:SetPoint("BOTTOMRIGHT", 0, 0)
		button.PullDownMenu.Button[i].Highlight:SetTexture(1, 1, 1, 0.2)
		button.PullDownMenu.Button[i]:SetHighlightTexture(button.PullDownMenu.Button[i].Highlight)

		if contentName == "TickSound" then
			button.PullDownMenu.Button[i].Text:SetText(soundsTick[i].name)
			button.PullDownMenu.Button[i].value1 = i
		elseif contentName == "TickChannel" then
			button.PullDownMenu.Button[i].Text:SetText(soundChannel[i].name)
			button.PullDownMenu.Button[i].value1 = i
		elseif contentName == "WarningSound" then
			button.PullDownMenu.Button[i].Text:SetText(soundsWarning[i].name)
			button.PullDownMenu.Button[i].value1 = i
		elseif contentName == "WarningChannelSound" then
			button.PullDownMenu.Button[i].Text:SetText(soundChannel[i].name)
			button.PullDownMenu.Button[i].value1 = i
		end
		button.PullDownMenu.Button[i]:Show()	

		if pulldownWidth == 0 then
			local w = button.PullDownMenu.Button[i].Text:GetStringWidth()+15+18
			if w > autoWidth then
				autoWidth = w
			end
		end
	end

	local newWidth = pulldownWidth
	if pulldownWidth == 0 then
		newWidth = autoWidth
	end

	button.PullDownButtonText:SetWidth(newWidth-sizeOffset-sizeOffset)
	button.PullDownMenu:SetWidth(newWidth)
	for i = 1, contentNum do
		button.PullDownMenu.Button[i]:SetWidth(newWidth-sizeOffset-sizeOffset)
		button.PullDownMenu.Button[i].Text:SetWidth(newWidth-sizeOffset-sizeOffset)
	end
	button:SetWidth(newWidth)

	button.PullDownMenu:SetScript("OnLeave", OnLeave)
	button.PullDownMenu:SetScript("OnHide", function(self) self:Hide() end) -- for esc close

	button:SetScript("OnLeave", OnLeave)
	button:SetScript("OnClick", function()
		if button.PullDownMenu:IsShown() then
			button.PullDownMenu:Hide()
			button.PullDownButtonExpand:SetTexCoord(unpack(Textures.Expand.coords))
		else
			button.PullDownMenu:Show()
			button.PullDownButtonExpand:SetTexCoord(unpack(Textures.Collapse.coords))
		end
	end)
end
-- Template PullDownMenu END ----------------------------------------

-- Template Slider START ----------------------------------------
TEMPLATE.DisableSlider = function(slider)
	slider.textMin:SetTextColor(0.5, 0.5, 0.5, 1)
	slider.textMax:SetTextColor(0.5, 0.5, 0.5, 1)
	slider.sliderBGL:SetTexCoord(unpack(Textures.SliderBG.coordsLdis))
	slider.sliderBGM:SetTexCoord(unpack(Textures.SliderBG.coordsMdis))
	slider.sliderBGR:SetTexCoord(unpack(Textures.SliderBG.coordsRdis))
	slider.thumb:SetTexCoord(0, 0, 0, 0)
	slider.Background:SetTexture(0, 0, 0, 0)
	slider:SetScript("OnEnter", NOOP)
	slider:SetScript("OnLeave", NOOP)
	slider:Disable()
end

TEMPLATE.EnableSlider = function(slider)
	slider.textMin:SetTextColor(0.8, 0.8, 0.8, 1)
	slider.textMax:SetTextColor(0.8, 0.8, 0.8, 1)
	slider.sliderBGL:SetTexCoord(unpack(Textures.SliderBG.coordsL))
	slider.sliderBGM:SetTexCoord(unpack(Textures.SliderBG.coordsM))
	slider.sliderBGR:SetTexCoord(unpack(Textures.SliderBG.coordsR))
	slider.thumb:SetTexCoord(unpack(Textures.SliderKnob.coords))
	slider:SetScript("OnEnter", function() slider.Background:SetTexture(1, 1, 1, 0.1) end)
	slider:SetScript("OnLeave", function() slider.Background:SetTexture(0, 0, 0, 0) end)
	slider:Enable()
end

TEMPLATE.Slider = function(slider, width, step, minVal, maxVal, curVal, func, measure)
	slider:SetWidth(width)
	slider:SetHeight(16)
	slider:SetValueStep(step) 
	slider:SetMinMaxValues(minVal, maxVal)
	slider:SetValue(curVal)
	slider:SetOrientation("HORIZONTAL")

	slider.Background = slider:CreateTexture(nil, "BACKGROUND")
	slider.Background:SetWidth(width)
	slider.Background:SetHeight(16)
	slider.Background:SetPoint("LEFT", 0, 0)
	slider.Background:SetTexture(0, 0, 0, 0)

	slider.textMin = slider:CreateFontString(nil, "ARTWORK", "GameFontNormalSmall")
	slider.textMin:SetPoint("TOP", slider, "BOTTOM", 0, -1)
	slider.textMin:SetPoint("LEFT", slider, "LEFT", 0, 0)
	slider.textMin:SetJustifyH("CENTER")
	slider.textMin:SetTextColor(0.8, 0.8, 0.8, 1)
	if measure == "%" then
		slider.textMin:SetText(minVal.."%")
	elseif measure == "K" then
		slider.textMin:SetText((minVal/1000).."k")
	elseif measure == "H" then
		slider.textMin:SetText((minVal/100))
	elseif measure == "px" then
		slider.textMin:SetText(minVal.."px")
	elseif measure == "blank" then
		slider.textMin:SetText("")
	else
		slider.textMin:SetText(minVal)
	end
	slider.textMax = slider:CreateFontString(nil, "ARTWORK", "GameFontNormalSmall")
	slider.textMax:SetPoint("TOP", slider, "BOTTOM", 0, -1)
	slider.textMax:SetPoint("RIGHT", slider, "RIGHT", 0, 0)
	slider.textMax:SetJustifyH("CENTER")
	slider.textMax:SetTextColor(0.8, 0.8, 0.8, 1)
	if measure == "%" then
		slider.textMax:SetText(maxVal.."%")
	elseif measure == "K" then
		slider.textMax:SetText((maxVal/1000).."k")
	elseif measure == "H" then
		slider.textMax:SetText((maxVal/100))
	elseif measure == "px" then
		slider.textMax:SetText(maxVal.."px")
	elseif measure == "blank" then
		slider.textMax:SetText("")
	else
		slider.textMax:SetText(maxVal)
	end

	slider.sliderBGL = slider:CreateTexture(nil, "BACKGROUND")
	slider.sliderBGL:SetWidth(5)
	slider.sliderBGL:SetHeight(6)
	slider.sliderBGL:SetPoint("LEFT", slider, "LEFT", 0, 0)
	slider.sliderBGL:SetTexture(Textures.DrainSoulerIcons.path)
	slider.sliderBGL:SetTexCoord(unpack(Textures.SliderBG.coordsL))
	slider.sliderBGM = slider:CreateTexture(nil, "BACKGROUND")
	slider.sliderBGM:SetWidth(width-5-5)
	slider.sliderBGM:SetHeight(6)
	slider.sliderBGM:SetPoint("LEFT", slider.sliderBGL, "RIGHT", 0, 0)
	slider.sliderBGM:SetTexture(Textures.DrainSoulerIcons.path)
	slider.sliderBGM:SetTexCoord(unpack(Textures.SliderBG.coordsM))
	slider.sliderBGR = slider:CreateTexture(nil, "BACKGROUND")
	slider.sliderBGR:SetWidth(5)
	slider.sliderBGR:SetHeight(6)
	slider.sliderBGR:SetPoint("LEFT", slider.sliderBGM, "RIGHT", 0, 0)
	slider.sliderBGR:SetTexture(Textures.DrainSoulerIcons.path)
	slider.sliderBGR:SetTexCoord(unpack(Textures.SliderBG.coordsR))

	slider.thumb = slider:CreateTexture(nil, "BORDER")
	slider.thumb:SetWidth(11)
	slider.thumb:SetHeight(17)
	slider.thumb:SetTexture(Textures.DrainSoulerIcons.path)
	slider.thumb:SetTexCoord(unpack(Textures.SliderKnob.coords))
	slider:SetThumbTexture(slider.thumb)

	slider:SetScript("OnValueChanged", function(self, value)
		if not slider:IsEnabled() then return end
		if func then
			func(self, value)
		end
	end)

	slider:SetScript("OnEnter", function() slider.Background:SetTexture(1, 1, 1, 0.1) end)
	slider:SetScript("OnLeave", function() slider.Background:SetTexture(0, 0, 0, 0) end)
end
-- Template Slider END ----------------------------------------
-- ---------------------------------------------------------------------------------------------------------------------



-- ---------------------------------------------------------------------------------------------------------------------
function DrainSouler:InitOptions()
	SlashCmdList["DRAINSOULER"] = function()
		if not GVAR.OptionsFrame then return end
		if GVAR.OptionsFrame:IsShown() then
			DrainSouler_Options.configMode = false
			DrainSouler:DisableConfigMode()
		else
			DrainSouler_Options.configMode = true
			DrainSouler:EnableConfigMode()
		end
	end
	SLASH_DRAINSOULER1 = "/ds"
	SLASH_DRAINSOULER2 = "/drainsouler"

	if DrainSouler_Options.version == nil then
		DrainSouler_Options.version = 4
	end
	if DrainSouler_Options.version == 1 then
		if DrainSouler_Options.TwentyFiveSound        then DrainSouler_Options.WarningSound        = DrainSouler_Options.TwentyFiveSound end
		if DrainSouler_Options.TwentyFiveSoundChannel then DrainSouler_Options.WarningChannelSound = DrainSouler_Options.TwentyFiveSoundChannel end
		if DrainSouler_Options.TwentyFiveSoundHP      then DrainSouler_Options.WarningHPSound      = DrainSouler_Options.TwentyFiveSoundHP end
		if DrainSouler_Options.TwentyFiveSoundHPP     then
			if DrainSouler_Options.TwentyFiveSoundHPP ~= 25 then
				print("|cffffff7fDrainSouler:|r Warning Sound option changed to 20%.")
				print("|cffffff7fDrainSouler:|r Please check options.")
			end
			DrainSouler_Options.WarningHPPSound = 20
		end
		DrainSouler_Options.TwentyFiveSound = nil
		DrainSouler_Options.TwentyFiveSoundChannel = nil
		DrainSouler_Options.TwentyFiveSoundHP = nil
		DrainSouler_Options.TwentyFiveSoundHPP = nil
		DrainSouler_Options.version = 2
	end
	if DrainSouler_Options.version == 2 then
		print("|cffffff7fDrainSouler Fix|r")
		print("|cffffff7fDrainSouler:|r The MainFrame parent is now UIParent. This may change the size and position.")
		print("|cffffff7fDrainSouler:|r Type /ds for configuration.")
		DrainSouler_Options.version = 3
	end
	if DrainSouler_Options.version == 3 then
		if DrainSouler_Options.TickSound then
			local old = DrainSouler_Options.TickSound
			DrainSouler_Options.TickSound = {}
			DrainSouler_Options.TickSound[1] = old
			DrainSouler_Options.TickSound[2] = old
			DrainSouler_Options.TickSound[3] = old
			DrainSouler_Options.TickSound[4] = old
		end
		if DrainSouler_Options.TickSoundChannel then
			local old = DrainSouler_Options.TickSoundChannel
			DrainSouler_Options.TickSoundChannel = nil
			DrainSouler_Options.TickChannel = {}
			DrainSouler_Options.TickChannel[1] = old
			DrainSouler_Options.TickChannel[2] = old
			DrainSouler_Options.TickChannel[3] = old
			DrainSouler_Options.TickChannel[4] = old
		end
		DrainSouler_Options.version = 4
	end

	if DrainSouler_Options.configMode          == nil then DrainSouler_Options.configMode          = true end
	if DrainSouler_Options.Lock                == nil then DrainSouler_Options.Lock                = false end
	if DrainSouler_Options.TalentDependent     == nil then DrainSouler_Options.TalentDependent     = false end
	if DrainSouler_Options.DetailInfo          == nil then DrainSouler_Options.DetailInfo          = false end
	if DrainSouler_Options.Size                == nil then DrainSouler_Options.Size                = 1 end
	if DrainSouler_Options.onlyInCombat        == nil then DrainSouler_Options.onlyInCombat        = true end
	if DrainSouler_Options.minHP               == nil then DrainSouler_Options.minHP               = 0 end
	if DrainSouler_Options.minHPP              == nil then DrainSouler_Options.minHPP              = 100 end
	if DrainSouler_Options.TickSound           == nil then DrainSouler_Options.TickSound           = {} end
	if DrainSouler_Options.TickSound[1]        == nil then DrainSouler_Options.TickSound[1]        = 1 end
	if DrainSouler_Options.TickSound[2]        == nil then DrainSouler_Options.TickSound[2]        = 1 end
	if DrainSouler_Options.TickSound[3]        == nil then DrainSouler_Options.TickSound[3]        = 1 end
	if DrainSouler_Options.TickSound[4]        == nil then DrainSouler_Options.TickSound[4]        = 1 end
	if DrainSouler_Options.TickChannel         == nil then DrainSouler_Options.TickChannel         = {} end
	if DrainSouler_Options.TickChannel[1]      == nil then DrainSouler_Options.TickChannel[1]      = 2 end
	if DrainSouler_Options.TickChannel[2]      == nil then DrainSouler_Options.TickChannel[2]      = 2 end
	if DrainSouler_Options.TickChannel[3]      == nil then DrainSouler_Options.TickChannel[3]      = 2 end
	if DrainSouler_Options.TickChannel[4]      == nil then DrainSouler_Options.TickChannel[4]      = 2 end
	if DrainSouler_Options.WarningSound        == nil then DrainSouler_Options.WarningSound        = 1 end
	if DrainSouler_Options.WarningChannelSound == nil then DrainSouler_Options.WarningChannelSound = 2 end
	if DrainSouler_Options.WarningHPSound      == nil then DrainSouler_Options.WarningHPSound      = 0 end
	if DrainSouler_Options.WarningHPPSound     == nil then DrainSouler_Options.WarningHPPSound     = 20 end
end
-- ---------------------------------------------------------------------------------------------------------------------

-- ---------------------------------------------------------------------------------------------------------------------
function DrainSouler:CreateInterfaceOptions()
	GVAR.InterfaceOptions = CreateFrame("Frame", "DrainSouler_InterfaceOptions")
	GVAR.InterfaceOptions.name = "DrainSouler"

	GVAR.InterfaceOptions.Title = GVAR.InterfaceOptions:CreateFontString(nil, "ARTWORK", "GameFontNormalLarge")
	GVAR.InterfaceOptions.Title:SetPoint("TOPLEFT", 16, -16)
	GVAR.InterfaceOptions.Title:SetText("DrainSouler")
	GVAR.InterfaceOptions.Title:SetJustifyH("LEFT")
	GVAR.InterfaceOptions.Title:SetJustifyV("TOP")

	GVAR.InterfaceOptions.OPTIONS = CreateFrame("Button", nil, DrainSouler_InterfaceOptions)
	TEMPLATE.TextButton(GVAR.InterfaceOptions.OPTIONS, L["Open Configuration"], 1)
	GVAR.InterfaceOptions.OPTIONS:SetPoint("TOPLEFT", GVAR.InterfaceOptions.Title, "BOTTOMLEFT", 0, -10)
	GVAR.InterfaceOptions.OPTIONS:SetWidth(200)
	GVAR.InterfaceOptions.OPTIONS:SetHeight(22)
	GVAR.InterfaceOptions.OPTIONS:SetScript("OnClick", function()
		InterfaceOptionsFrame_Show()
		HideUIPanel(GameMenuFrame)
		DrainSouler_Options.configMode = true
		DrainSouler:EnableConfigMode()
	end)

	InterfaceOptions_AddCategory(GVAR.InterfaceOptions)
end
-- ---------------------------------------------------------------------------------------------------------------------

-- ---------------------------------------------------------------------------------------------------------------------
function DrainSouler:CreateOptionsFrame()
	local frameWidth  = 460
	local frameHeight = 600
	local fw = 0

	GVAR.OptionsFrame = CreateFrame("Frame", nil, UIParent)
	TEMPLATE.BorderTRBL(GVAR.OptionsFrame)
	GVAR.OptionsFrame:EnableMouse(true)
	GVAR.OptionsFrame:SetMovable(true)
	GVAR.OptionsFrame:SetToplevel(true)
	GVAR.OptionsFrame:SetClampedToScreen(true)
	GVAR.OptionsFrame:SetClampRectInsets((frameWidth-50)/2, -((frameWidth-50)/2), -(frameHeight-35), frameHeight-35)
	GVAR.OptionsFrame:SetWidth(frameWidth)
	GVAR.OptionsFrame:SetHeight(frameHeight)
	GVAR.OptionsFrame:SetPoint("CENTER")
	GVAR.OptionsFrame:Hide()
	GVAR.OptionsFrame:SetScript("OnMouseWheel", NOOP)

	-- spell info
	for i = 1, 4 do
		             local spellname = DrainSoulName
		    if i == 2 then spellname = DrainLifeName
		elseif i == 3 then spellname = HarvestLifeName
		elseif i == 4 then spellname = MaleficGraspName end

		GVAR.OptionsFrame["ChanSpell"..i] = CreateFrame("Button", nil, GVAR.OptionsFrame)
		if i == 1 then
			GVAR.OptionsFrame["ChanSpell"..i]:SetPoint("TOPRIGHT", GVAR.OptionsFrame, "TOPRIGHT", -5, -5)
		else
			GVAR.OptionsFrame["ChanSpell"..i]:SetPoint("TOP", GVAR.OptionsFrame["ChanSpell"..(i-1)], "BOTTOM", 0, -2)
		end
		GVAR.OptionsFrame["ChanSpell"..i]:SetWidth(32)
		GVAR.OptionsFrame["ChanSpell"..i]:SetHeight(32)
		GVAR.OptionsFrame["ChanSpell"..i.."BG"] = GVAR.OptionsFrame["ChanSpell"..i]:CreateTexture(nil, "BACKGROUND")
		GVAR.OptionsFrame["ChanSpell"..i.."BG"]:SetAllPoints(GVAR.OptionsFrame["ChanSpell"..i])
		GVAR.OptionsFrame["ChanSpell"..i.."BG"]:SetTexture(channelSpells[spellname].timerColor[1], channelSpells[spellname].timerColor[2], channelSpells[spellname].timerColor[3], 0.3)
		GVAR.OptionsFrame["ChanSpell"..i.."Border"] = GVAR.OptionsFrame["ChanSpell"..i]:CreateTexture(nil, "BORDER")
		GVAR.OptionsFrame["ChanSpell"..i.."Border"]:SetWidth(30)
		GVAR.OptionsFrame["ChanSpell"..i.."Border"]:SetHeight(30)
		GVAR.OptionsFrame["ChanSpell"..i.."Border"]:SetPoint("TOPLEFT", GVAR.OptionsFrame["ChanSpell"..i], "TOPLEFT", 1, -1)
		GVAR.OptionsFrame["ChanSpell"..i.."Border"]:SetTexture(0, 0, 0, 1)
		GVAR.OptionsFrame["ChanSpell"..i.."Texture"] = GVAR.OptionsFrame["ChanSpell"..i]:CreateTexture(nil, "ARTWORK")
		GVAR.OptionsFrame["ChanSpell"..i.."Texture"]:SetWidth(28)
		GVAR.OptionsFrame["ChanSpell"..i.."Texture"]:SetHeight(28)
		GVAR.OptionsFrame["ChanSpell"..i.."Texture"]:SetPoint("TOPLEFT", GVAR.OptionsFrame["ChanSpell"..i], "TOPLEFT", 2, -2)
		GVAR.OptionsFrame["ChanSpell"..i.."Texture"]:SetTexture(channelSpells[spellname].spellIcon)
		GVAR.OptionsFrame["ChanSpell"..i.."Texture"]:SetTexCoord(6/64, 58/64, 6/64, 58/64)

		GVAR.OptionsFrame["ChanSpell"..i]:SetScript("OnEnter", function()
			GameTooltip:SetOwner(GVAR.OptionsFrame["ChanSpell"..i], "ANCHOR_BOTTOMLEFT", 0, 32)
			GameTooltip:AddLine(spellname, channelSpells[spellname].timerColor[1], channelSpells[spellname].timerColor[2], channelSpells[spellname].timerColor[3])
			GameTooltip:AddLine("spell-ID: "..channelSpells[spellname].spellID, 1, 1, 1)
			GameTooltip:AddLine("spellDura: "..channelSpells[spellname].spellDura.."ms", 1, 1, 1)
			GameTooltip:AddLine("baseTickDura: "..channelSpells[spellname].baseTickDura.."ms", 1, 1, 1)
			GameTooltip:Show()

			GVAR.OptionsFrame["ChanSpell"..i.."BG"]:SetTexture(channelSpells[spellname].timerColor[1], channelSpells[spellname].timerColor[2], channelSpells[spellname].timerColor[3], 1)

			GVAR.MainFrame.TickTimerTexture:SetTexture(channelSpells[spellname].timerColor[1], channelSpells[spellname].timerColor[2], channelSpells[spellname].timerColor[3], 1)
			GVAR.MainFrame.TickTimerTexture:SetWidth(sizeTimerWidth)
			GVAR.MainFrame.TickTimer:Show()
			GVAR.MainFrame.ChannelTimerTexture:SetTexture(channelSpells[spellname].timerColor[1], channelSpells[spellname].timerColor[2], channelSpells[spellname].timerColor[3], 1)
			GVAR.MainFrame.ChannelTimerTexture:SetWidth(sizeTimerWidth)
			GVAR.MainFrame.ChannelTimer:Show()
			GVAR.MainFrame.SpellIconTexture:SetTexture(channelSpells[spellname].spellIcon)
			GVAR.MainFrame.SpellIcon:Show()
			if spellname ~= DrainSoulName then
				GVAR.MainFrame.WarningIcon:Hide()
			end
		end)

		GVAR.OptionsFrame["ChanSpell"..i]:SetScript("OnLeave", function()
			GameTooltip:Hide()

			GVAR.OptionsFrame["ChanSpell"..i.."BG"]:SetTexture(channelSpells[spellname].timerColor[1], channelSpells[spellname].timerColor[2], channelSpells[spellname].timerColor[3], 0.3)

			GVAR.MainFrame.TickTimer:Hide()
			GVAR.MainFrame.ChannelTimer:Hide()
			GVAR.MainFrame.SpellIconTexture:SetTexture(channelSpells[DrainSoulName].spellIcon)
			GVAR.MainFrame.SpellIcon:Show()
			GVAR.MainFrame.WarningIcon:Show()
		end)
	end
	GVAR.OptionsFrame.Info = GVAR.OptionsFrame:CreateFontString(nil, "ARTWORK", "GameFontNormalSmall")
	GVAR.OptionsFrame.Info:SetPoint("TOPRIGHT", GVAR.OptionsFrame["ChanSpell1"], "TOPLEFT", -5, 0)
	GVAR.OptionsFrame.Info:SetJustifyH("RIGHT")
	GVAR.OptionsFrame.Info:SetText(INFO..":")
	GVAR.OptionsFrame.Info:SetTextColor(1, 1, 1, 1)

	-- Title
	GVAR.OptionsFrame.Title = GVAR.OptionsFrame:CreateFontString(nil, "ARTWORK", "GameFontNormalLarge")
	GVAR.OptionsFrame.Title:SetPoint("TOPLEFT", GVAR.OptionsFrame, "TOPLEFT", 10, -10)
	GVAR.OptionsFrame.Title:SetJustifyH("LEFT")
	GVAR.OptionsFrame.Title:SetText("DrainSouler "..L["Configuration"])
	fw = fw + 10 + GVAR.OptionsFrame.Title:GetStringHeight()

	-- MoveNote
	GVAR.OptionsFrame.MoveNote = GVAR.OptionsFrame:CreateFontString(nil, "ARTWORK", "GameFontNormalSmall")
	GVAR.OptionsFrame.MoveNote:SetPoint("LEFT", GVAR.OptionsFrame, "LEFT", 10, 0)
	GVAR.OptionsFrame.MoveNote:SetPoint("TOP", GVAR.OptionsFrame.Title, "BOTTOM", 0, -10)
	GVAR.OptionsFrame.MoveNote:SetJustifyH("LEFT")
	GVAR.OptionsFrame.MoveNote:SetText(L["Click and Drag on DrainSouler Frame to move it."])
	fw = fw + 10 + GVAR.OptionsFrame.MoveNote:GetStringHeight()

	-- Lock
	GVAR.OptionsFrame.Lock = CreateFrame("CheckButton", nil, GVAR.OptionsFrame)
	TEMPLATE.CheckButton(GVAR.OptionsFrame.Lock, 16, 4, L["Lock DrainSouler"].." |cffaaaaaa("..L["enables click through frame"]..")|r")
	GVAR.OptionsFrame.Lock:SetPoint("LEFT", GVAR.OptionsFrame, "LEFT", 10, 0)
	GVAR.OptionsFrame.Lock:SetPoint("TOP", GVAR.OptionsFrame.MoveNote, "BOTTOM", 0, -10)
	GVAR.OptionsFrame.Lock:SetChecked(DrainSouler_Options.Lock)
	TEMPLATE.EnableCheckButton(GVAR.OptionsFrame.Lock)
	GVAR.OptionsFrame.Lock:SetScript("OnClick", function(self)
		DrainSouler_Options.Lock = not DrainSouler_Options.Lock
		if DrainSouler_Options.Lock then
			GVAR.OptionsFrame.MoveNote:SetTextColor(0, 0, 0, 1)
			DrainSouler:DisableMovement()
		else
			GVAR.OptionsFrame.MoveNote:SetTextColor(1, 1, 0.5, 1)
			DrainSouler:EnableMovement()
		end
	end)
	fw = fw + 10 + 16

	-- onlyInCombat
	GVAR.OptionsFrame.onlyInCombat = CreateFrame("CheckButton", nil, GVAR.OptionsFrame)
	TEMPLATE.CheckButton(GVAR.OptionsFrame.onlyInCombat, 16, 4, L["Show DrainSouler in combat only."])
	GVAR.OptionsFrame.onlyInCombat:SetPoint("LEFT", GVAR.OptionsFrame, "LEFT", 10, 0)
	GVAR.OptionsFrame.onlyInCombat:SetPoint("TOP", GVAR.OptionsFrame.Lock, "BOTTOM", 0, -10)
	GVAR.OptionsFrame.onlyInCombat:SetChecked(DrainSouler_Options.onlyInCombat)
	TEMPLATE.EnableCheckButton(GVAR.OptionsFrame.onlyInCombat)
	GVAR.OptionsFrame.onlyInCombat:SetScript("OnClick", function(self)
		DrainSouler_Options.onlyInCombat = not DrainSouler_Options.onlyInCombat
	end)
	fw = fw + 10 + 16

	-- TalentDependent
	GVAR.OptionsFrame.TalentDependent = CreateFrame("CheckButton", nil, GVAR.OptionsFrame)
	TEMPLATE.CheckButton(GVAR.OptionsFrame.TalentDependent, 16, 4, L["Deactivate DrainSouler if your specialization is not Affliction."])
	GVAR.OptionsFrame.TalentDependent:SetPoint("LEFT", GVAR.OptionsFrame, "LEFT", 10, 0)
	GVAR.OptionsFrame.TalentDependent:SetPoint("TOP", GVAR.OptionsFrame.onlyInCombat, "BOTTOM", 0, -10)
	GVAR.OptionsFrame.TalentDependent:SetChecked(DrainSouler_Options.TalentDependent)
	TEMPLATE.EnableCheckButton(GVAR.OptionsFrame.TalentDependent)
	GVAR.OptionsFrame.TalentDependent:SetScript("OnClick", function(self)
		DrainSouler_Options.TalentDependent = not DrainSouler_Options.TalentDependent
		DrainSouler:UpdateTalentDisplay()
	end)
	fw = fw + 10 + 16

	GVAR.OptionsFrame.TalentDependentDescription = GVAR.OptionsFrame:CreateFontString(nil, "ARTWORK", "GameFontNormalSmall")
	GVAR.OptionsFrame.TalentDependentDescription:SetPoint("LEFT", GVAR.OptionsFrame, "LEFT", 30, 0)
	GVAR.OptionsFrame.TalentDependentDescription:SetPoint("TOP", GVAR.OptionsFrame.TalentDependent, "BOTTOM", 0, -5)
	GVAR.OptionsFrame.TalentDependentDescription:SetJustifyH("LEFT")
	GVAR.OptionsFrame.TalentDependentDescription:SetText( L["Your current Spec:"] )
	GVAR.OptionsFrame.TalentDependentDescription:SetTextColor(0.6, 0.6, 0.6, 1)
	fw = fw + 5 + GVAR.OptionsFrame.TalentDependentDescription:GetStringHeight()

	-- DetailInfo
	GVAR.OptionsFrame.DetailInfo = CreateFrame("CheckButton", nil, GVAR.OptionsFrame)
	TEMPLATE.CheckButton(GVAR.OptionsFrame.DetailInfo, 16, 4, L["Show detailed information."])
	GVAR.OptionsFrame.DetailInfo:SetPoint("LEFT", GVAR.OptionsFrame, "LEFT", 10, 0)
	GVAR.OptionsFrame.DetailInfo:SetPoint("TOP", GVAR.OptionsFrame.TalentDependentDescription, "BOTTOM", 0, -10)
	GVAR.OptionsFrame.DetailInfo:SetChecked(DrainSouler_Options.DetailInfo)
	TEMPLATE.EnableCheckButton(GVAR.OptionsFrame.DetailInfo)
	GVAR.OptionsFrame.DetailInfo:SetScript("OnClick", function(self)
		DrainSouler_Options.DetailInfo = not DrainSouler_Options.DetailInfo
		if DrainSouler_Options.DetailInfo then
			GVAR.MainFrame.DurationText:SetText("numTicks | tick duration in ms | complete duration in ms")
			GVAR.MainFrame.ModifiersText:SetText(STAT_HASTE.." |cffffff82+x%|r | "..STAT_SPELLPOWER.." |cffffff82+x%|r")
		else
			GVAR.MainFrame.DurationText:SetText("")
			GVAR.MainFrame.ModifiersText:SetText("")
		end
	end)
	fw = fw + 10 + 16

	-- Size
	GVAR.OptionsFrame.SizeTitle = GVAR.OptionsFrame:CreateFontString(nil, "ARTWORK", "GameFontNormalSmall")
	GVAR.OptionsFrame.SizeTitle:SetPoint("LEFT", GVAR.OptionsFrame, "LEFT", 10, 0)
	GVAR.OptionsFrame.SizeTitle:SetPoint("TOP", GVAR.OptionsFrame.DetailInfo, "BOTTOM", 0, -10)
	GVAR.OptionsFrame.SizeTitle:SetJustifyH("LEFT")
	GVAR.OptionsFrame.SizeTitle:SetText(L["Size"]..": |cffffff99"..(DrainSouler_Options.Size*100).."%|r")
	GVAR.OptionsFrame.SizeTitle:SetTextColor(1, 1, 1, 1)
	fw = fw + 10 + GVAR.OptionsFrame.SizeTitle:GetStringHeight()

	GVAR.OptionsFrame.Size = CreateFrame("Slider", nil, GVAR.OptionsFrame)
	TEMPLATE.Slider(
		GVAR.OptionsFrame.Size,
		150,
		5,
		50,
		200,
		DrainSouler_Options.Size*100,
		function(self, value)--SizeSliderFunc
			DrainSouler_Options.Size = value/100
			GVAR.OptionsFrame.Size:SetValue(DrainSouler_Options.Size*100)
			GVAR.OptionsFrame.SizeTitle:SetText(L["Size"]..": |cffffff99"..(DrainSouler_Options.Size*100).."%|r")
			DrainSouler:SetupSize()
		end,
		"%"
	)
	GVAR.OptionsFrame.Size:SetPoint("LEFT", GVAR.OptionsFrame, "LEFT", 20, 0)
	GVAR.OptionsFrame.Size:SetPoint("TOP", GVAR.OptionsFrame.SizeTitle, "BOTTOM", 0, -5)
	fw = fw + 5 + 17
	fw = fw + 1 + GVAR.OptionsFrame.Size.textMin:GetStringHeight()

	-- minHP
	GVAR.OptionsFrame.minHPTitle = GVAR.OptionsFrame:CreateFontString(nil, "ARTWORK", "GameFontNormalSmall")
	GVAR.OptionsFrame.minHPTitle:SetPoint("LEFT", GVAR.OptionsFrame, "LEFT", 10, 0)
	GVAR.OptionsFrame.minHPTitle:SetPoint("TOP", GVAR.OptionsFrame.Size.textMin, "BOTTOM", 0, -15)
	GVAR.OptionsFrame.minHPTitle:SetJustifyH("LEFT")
	GVAR.OptionsFrame.minHPTitle:SetText(L["Target HP"]..": |cffffff99"..(DrainSouler_Options.minHP/1000).."K|r")
	GVAR.OptionsFrame.minHPTitle:SetTextColor(1, 1, 1, 1)
	fw = fw + 15 + GVAR.OptionsFrame.minHPTitle:GetStringHeight()

	GVAR.OptionsFrame.minHP = CreateFrame("Slider", nil, GVAR.OptionsFrame)
	TEMPLATE.Slider(
		GVAR.OptionsFrame.minHP,
		200,
		10000,
		0,
		1000000,
		DrainSouler_Options.minHP,
		function(self, value)--minHPSliderFunc
			DrainSouler_Options.minHP = value
			GVAR.OptionsFrame.minHP:SetValue(DrainSouler_Options.minHP)
			GVAR.OptionsFrame.minHPTitle:SetText(L["Target HP"]..": |cffffff99"..(DrainSouler_Options.minHP/1000).."K|r")
			GVAR.OptionsFrame.minHPDescription:SetText( string.format(L["Shows DrainSouler if your target is at or below %s health and the maximum health is greater than %s."], "|cffffff99"..DrainSouler_Options.minHPP.."%|r", "|cffffff99"..(DrainSouler_Options.minHP/1000).."K|r") )
		end,
		"K"
	)
	GVAR.OptionsFrame.minHP:SetPoint("LEFT", GVAR.OptionsFrame, "LEFT", 20, 0)
	GVAR.OptionsFrame.minHP:SetPoint("TOP", GVAR.OptionsFrame.minHPTitle, "BOTTOM", 0, -5)
	fw = fw + 5 + 17
	fw = fw + 1 + GVAR.OptionsFrame.minHP.textMin:GetStringHeight()

	-- minHPP
	GVAR.OptionsFrame.minHPPTitle = GVAR.OptionsFrame:CreateFontString(nil, "ARTWORK", "GameFontNormalSmall")
	GVAR.OptionsFrame.minHPPTitle:SetPoint("LEFT", GVAR.OptionsFrame.minHP, "RIGHT", 20, 0)
	GVAR.OptionsFrame.minHPPTitle:SetPoint("TOP", GVAR.OptionsFrame.minHPTitle, "TOP", 0, 0)
	GVAR.OptionsFrame.minHPPTitle:SetJustifyH("LEFT")
	GVAR.OptionsFrame.minHPPTitle:SetText(L["Target HP (%)"]..": |cffffff99"..DrainSouler_Options.minHPP.."%|r")
	GVAR.OptionsFrame.minHPPTitle:SetTextColor(1, 1, 1, 1)
	fw = fw + 0

	GVAR.OptionsFrame.minHPP = CreateFrame("Slider", nil, GVAR.OptionsFrame)
	TEMPLATE.Slider(
		GVAR.OptionsFrame.minHPP,
		150,
		1,
		20,
		100,
		DrainSouler_Options.minHPP,
		function(self, value)--minHPPSliderFunc
			DrainSouler_Options.minHPP = value
			GVAR.OptionsFrame.minHPP:SetValue(DrainSouler_Options.minHPP)
			GVAR.OptionsFrame.minHPPTitle:SetText(L["Target HP (%)"]..": |cffffff99"..DrainSouler_Options.minHPP.."%|r")
			GVAR.OptionsFrame.minHPDescription:SetText( string.format(L["Shows DrainSouler if your target is at or below %s health and the maximum health is greater than %s."], "|cffffff99"..DrainSouler_Options.minHPP.."%|r", "|cffffff99"..(DrainSouler_Options.minHP/1000).."K|r") )
		end,
		"%"
	)
	GVAR.OptionsFrame.minHPP:SetPoint("TOPLEFT", GVAR.OptionsFrame.minHPPTitle, "BOTTOMLEFT", 0, -5)
	fw = fw + 0

	GVAR.OptionsFrame.minHPDescription = GVAR.OptionsFrame:CreateFontString(nil, "ARTWORK", "GameFontNormalSmall")
	GVAR.OptionsFrame.minHPDescription:SetPoint("LEFT", GVAR.OptionsFrame, "LEFT", 20, 0)
	GVAR.OptionsFrame.minHPDescription:SetPoint("TOP", GVAR.OptionsFrame.minHP.textMin, "BOTTOM", 0, -5)
	GVAR.OptionsFrame.minHPDescription:SetWidth(frameWidth-40)
	GVAR.OptionsFrame.minHPDescription:SetJustifyH("LEFT")
	GVAR.OptionsFrame.minHPDescription:SetText( string.format(L["Shows DrainSouler if your target is at or below %s health and the maximum health is greater than %s."], "|cffffff99"..DrainSouler_Options.minHPP.."%|r", "|cffffff99"..(DrainSouler_Options.minHP/1000).."K|r") )
	GVAR.OptionsFrame.minHPDescription:SetTextColor(0.6, 0.6, 0.6, 1)
	fw = fw + 5 + GVAR.OptionsFrame.minHPDescription:GetStringHeight()

	-- TickSound
	GVAR.OptionsFrame.TickSoundTitle = GVAR.OptionsFrame:CreateFontString(nil, "ARTWORK", "GameFontNormalSmall")
	GVAR.OptionsFrame.TickSoundTitle:SetPoint("LEFT", GVAR.OptionsFrame, "LEFT", 10, 0)
	GVAR.OptionsFrame.TickSoundTitle:SetPoint("TOP", GVAR.OptionsFrame.minHPDescription, "BOTTOM", 0, -15)
	GVAR.OptionsFrame.TickSoundTitle:SetJustifyH("LEFT")
	GVAR.OptionsFrame.TickSoundTitle:SetText(L["Tick Sound"]..":")
	GVAR.OptionsFrame.TickSoundTitle:SetTextColor(1, 1, 1, 1)
	fw = fw + 15 + GVAR.OptionsFrame.minHPPTitle:GetStringHeight()

	for i = 1, #channelINTspells do
		if not GVAR.OptionsFrame.TickSoundImg then GVAR.OptionsFrame.TickSoundImg = {} end
		GVAR.OptionsFrame.TickSoundImg[i] = CreateFrame("Button", nil, GVAR.OptionsFrame)
		if i == 1 then
			GVAR.OptionsFrame.TickSoundImg[i]:SetPoint("LEFT", GVAR.OptionsFrame, "LEFT", 20, 0)
			GVAR.OptionsFrame.TickSoundImg[i]:SetPoint("TOP", GVAR.OptionsFrame.TickSoundTitle, "BOTTOM", 0, -5)
			fw = fw + 5 + 24
		else
			GVAR.OptionsFrame.TickSoundImg[i]:SetPoint("TOP", GVAR.OptionsFrame.TickSoundImg[(i-1)], "BOTTOM", 0, -2)
			fw = fw + 2 + 24
		end
		GVAR.OptionsFrame.TickSoundImg[i]:SetWidth(24)
		GVAR.OptionsFrame.TickSoundImg[i]:SetHeight(24)
		GVAR.OptionsFrame.TickSoundImg[i].BG = GVAR.OptionsFrame.TickSoundImg[i]:CreateTexture(nil, "BACKGROUND")
		GVAR.OptionsFrame.TickSoundImg[i].BG:SetAllPoints(GVAR.OptionsFrame.TickSoundImg[i])
		GVAR.OptionsFrame.TickSoundImg[i].BG:SetTexture(channelSpells[ channelINTspells[i] ].timerColor[1], channelSpells[ channelINTspells[i] ].timerColor[2], channelSpells[ channelINTspells[i] ].timerColor[3], 1)
		GVAR.OptionsFrame.TickSoundImg[i].Border = GVAR.OptionsFrame.TickSoundImg[i]:CreateTexture(nil, "BORDER")
		GVAR.OptionsFrame.TickSoundImg[i].Border:SetWidth(22)
		GVAR.OptionsFrame.TickSoundImg[i].Border:SetHeight(22)
		GVAR.OptionsFrame.TickSoundImg[i].Border:SetPoint("TOPLEFT", GVAR.OptionsFrame.TickSoundImg[i], "TOPLEFT", 1, -1)
		GVAR.OptionsFrame.TickSoundImg[i].Border:SetTexture(0, 0, 0, 1)
		GVAR.OptionsFrame.TickSoundImg[i].Texture = GVAR.OptionsFrame.TickSoundImg[i]:CreateTexture(nil, "ARTWORK")
		GVAR.OptionsFrame.TickSoundImg[i].Texture:SetWidth(20)
		GVAR.OptionsFrame.TickSoundImg[i].Texture:SetHeight(20)
		GVAR.OptionsFrame.TickSoundImg[i].Texture:SetPoint("TOPLEFT", GVAR.OptionsFrame.TickSoundImg[i], "TOPLEFT", 2, -2)
		GVAR.OptionsFrame.TickSoundImg[i].Texture:SetTexture(channelSpells[ channelINTspells[i] ].spellIcon)
		GVAR.OptionsFrame.TickSoundImg[i].Texture:SetTexCoord(6/64, 58/64, 6/64, 58/64)

		if not GVAR.OptionsFrame.TickSoundPullDown then GVAR.OptionsFrame.TickSoundPullDown = {} end
		GVAR.OptionsFrame.TickSoundPullDown[i] = CreateFrame("Button", nil, GVAR.OptionsFrame)
		TEMPLATE.PullDownMenu(
			GVAR.OptionsFrame.TickSoundPullDown[i],
			"TickSound",
			soundsTick[ DrainSouler_Options.TickSound[i] ].name,
			0,
			#soundsTick,
			function(value)--TickSoundPullDownFunc
				DrainSouler_Options.TickSound[i] = value
				if DrainSouler_Options.TickSound[i] > 1 then
					TEMPLATE.EnablePullDownMenu(GVAR.OptionsFrame.TickChannelPullDown[i])
					GVAR.OptionsFrame.TickChannelTitle[i]:SetTextColor(1, 1, 1, 1)
					PlaySoundFile(soundsTick[ DrainSouler_Options.TickSound[i] ].path, soundChannel[ DrainSouler_Options.TickChannel[i] ].channel)
				else
					TEMPLATE.DisablePullDownMenu(GVAR.OptionsFrame.TickChannelPullDown[i])
					GVAR.OptionsFrame.TickChannelTitle[i]:SetTextColor(0.5, 0.5, 0.5, 1)
				end
			end
		)
		GVAR.OptionsFrame.TickSoundPullDown[i]:SetPoint("LEFT", GVAR.OptionsFrame.TickSoundImg[i], "RIGHT", 10, 0)
		GVAR.OptionsFrame.TickSoundPullDown[i]:SetHeight(18)
		TEMPLATE.EnablePullDownMenu(GVAR.OptionsFrame.TickSoundPullDown[i])

		if not GVAR.OptionsFrame.TickChannelTitle then GVAR.OptionsFrame.TickChannelTitle = {} end
		GVAR.OptionsFrame.TickChannelTitle[i] = GVAR.OptionsFrame:CreateFontString(nil, "ARTWORK", "GameFontNormalSmall")
		GVAR.OptionsFrame.TickChannelTitle[i]:SetPoint("LEFT", GVAR.OptionsFrame.TickSoundPullDown[i], "RIGHT", 10, 0)
		GVAR.OptionsFrame.TickChannelTitle[i]:SetJustifyH("LEFT")
		GVAR.OptionsFrame.TickChannelTitle[i]:SetText(CHANNEL..": ")
		GVAR.OptionsFrame.TickChannelTitle[i]:SetTextColor(1, 1, 1, 1)

		if not GVAR.OptionsFrame.TickChannelPullDown then GVAR.OptionsFrame.TickChannelPullDown = {} end
		GVAR.OptionsFrame.TickChannelPullDown[i] = CreateFrame("Button", nil, GVAR.OptionsFrame)
		TEMPLATE.PullDownMenu(
			GVAR.OptionsFrame.TickChannelPullDown[i],
			"TickChannel",
			soundChannel[ DrainSouler_Options.TickChannel[i] ].name,
			0,
			#soundChannel,
			function(value)--TickChannelPullDownFunc
				DrainSouler_Options.TickChannel[i] = value
				if DrainSouler_Options.TickSound[i] > 1 then
					PlaySoundFile(soundsTick[ DrainSouler_Options.TickSound[i] ].path, soundChannel[ DrainSouler_Options.TickChannel[i] ].channel)
				end
			end
		)
		GVAR.OptionsFrame.TickChannelPullDown[i]:SetPoint("LEFT", GVAR.OptionsFrame.TickChannelTitle[i], "RIGHT", 5, 0)
		GVAR.OptionsFrame.TickChannelPullDown[i]:SetHeight(18)
		if DrainSouler_Options.TickSound[i] > 1 then
			TEMPLATE.EnablePullDownMenu(GVAR.OptionsFrame.TickChannelPullDown[i])
			GVAR.OptionsFrame.TickChannelTitle[i]:SetTextColor(1, 1, 1, 1)
		else
			TEMPLATE.DisablePullDownMenu(GVAR.OptionsFrame.TickChannelPullDown[i])
			GVAR.OptionsFrame.TickChannelTitle[i]:SetTextColor(0.5, 0.5, 0.5, 1)
		end
	end

	GVAR.OptionsFrame.TickSoundDescription = GVAR.OptionsFrame:CreateFontString(nil, "ARTWORK", "GameFontNormalSmall")
	GVAR.OptionsFrame.TickSoundDescription:SetPoint("LEFT", GVAR.OptionsFrame, "LEFT", 20, 0)
	GVAR.OptionsFrame.TickSoundDescription:SetPoint("TOP", GVAR.OptionsFrame.TickSoundImg[ #channelINTspells ], "BOTTOM", 0, -5)
	GVAR.OptionsFrame.TickSoundDescription:SetJustifyH("LEFT")
	GVAR.OptionsFrame.TickSoundDescription:SetText(L["Plays a sound when a channeling spell dealt damage."])
	GVAR.OptionsFrame.TickSoundDescription:SetTextColor(0.6, 0.6, 0.6, 1)
	fw = fw + 5 + GVAR.OptionsFrame.TickSoundDescription:GetStringHeight()

	-- WarningSound
	GVAR.OptionsFrame.WarningSoundTitle = GVAR.OptionsFrame:CreateFontString(nil, "ARTWORK", "GameFontNormalSmall")
	GVAR.OptionsFrame.WarningSoundTitle:SetPoint("LEFT", GVAR.OptionsFrame, "LEFT", 10, 0)
	GVAR.OptionsFrame.WarningSoundTitle:SetPoint("TOP", GVAR.OptionsFrame.TickSoundDescription, "BOTTOM", 0, -15)
	GVAR.OptionsFrame.WarningSoundTitle:SetJustifyH("LEFT")
	GVAR.OptionsFrame.WarningSoundTitle:SetText(L["Warning Sound"]..": |cffffff99"..soundsWarning[DrainSouler_Options.WarningSound].name.."|r")
	GVAR.OptionsFrame.WarningSoundTitle:SetTextColor(1, 1, 1, 1)
	fw = fw + 15 + GVAR.OptionsFrame.WarningSoundTitle:GetStringHeight()

	GVAR.OptionsFrame.WarningSoundPullDown = CreateFrame("Button", nil, GVAR.OptionsFrame)
	TEMPLATE.PullDownMenu(
		GVAR.OptionsFrame.WarningSoundPullDown,
		"WarningSound",
		soundsWarning[DrainSouler_Options.WarningSound].name,
		0,
		#soundsWarning,
		function(value)--WarningSoundPullDownFunc
			DrainSouler_Options.WarningSound = value
			GVAR.OptionsFrame.WarningSoundTitle:SetText(L["Warning Sound"]..": |cffffff99"..soundsWarning[DrainSouler_Options.WarningSound].name.."|r")
			if DrainSouler_Options.WarningSound > 1 then
				TEMPLATE.EnableSlider(GVAR.OptionsFrame.WarningHPSound)
				TEMPLATE.EnableSlider(GVAR.OptionsFrame.WarningHPPSound)
				GVAR.OptionsFrame.WarningHPSoundTitle:SetTextColor(1, 1, 1, 1)
				GVAR.OptionsFrame.WarningHPPSoundTitle:SetTextColor(1, 1, 1, 1)
				GVAR.OptionsFrame.WarningSoundDescription:SetTextColor(0.6, 0.6, 0.6, 1)
				GVAR.OptionsFrame.WarningSoundDescription:SetText( string.format(L["Plays a sound if your target is at or below %s health and the maximum health is greater than %s."], "|cffffff99"..DrainSouler_Options.WarningHPPSound.."%|r", "|cffffff99"..(DrainSouler_Options.WarningHPSound/1000).."K|r") )
				GVAR.OptionsFrame.WarningSoundWarning:SetTextColor(0.5, 0.5, 1, 1) -- WaRN
				TEMPLATE.EnablePullDownMenu(GVAR.OptionsFrame.WarningChannelSoundPullDown)
				GVAR.OptionsFrame.WarningChannelSoundTitle:SetTextColor(1, 1, 1, 1)
				PlaySoundFile(soundsWarning[value].path, soundChannel[DrainSouler_Options.WarningChannelSound].channel)
			else
				TEMPLATE.DisableSlider(GVAR.OptionsFrame.WarningHPSound)
				TEMPLATE.DisableSlider(GVAR.OptionsFrame.WarningHPPSound)
				GVAR.OptionsFrame.WarningHPSoundTitle:SetTextColor(0.5, 0.5, 0.5, 1)
				GVAR.OptionsFrame.WarningHPPSoundTitle:SetTextColor(0.5, 0.5, 0.5, 1)
				GVAR.OptionsFrame.WarningSoundDescription:SetTextColor(0.5, 0.5, 0.5, 1)
				GVAR.OptionsFrame.WarningSoundDescription:SetText( string.format(L["Plays a sound if your target is at or below %s health and the maximum health is greater than %s."], "|cffffff99"..DrainSouler_Options.WarningHPPSound.."%|r", "|cffffff99"..(DrainSouler_Options.WarningHPSound/1000).."K|r") )
				GVAR.OptionsFrame.WarningSoundWarning:SetTextColor(0.5, 0.5, 0.5, 1)
				TEMPLATE.DisablePullDownMenu(GVAR.OptionsFrame.WarningChannelSoundPullDown)
				GVAR.OptionsFrame.WarningChannelSoundTitle:SetTextColor(0.5, 0.5, 0.5, 1)
			end
		end
	)
	GVAR.OptionsFrame.WarningSoundPullDown:SetPoint("LEFT", GVAR.OptionsFrame, "LEFT", 20, 0)
	GVAR.OptionsFrame.WarningSoundPullDown:SetPoint("TOP", GVAR.OptionsFrame.WarningSoundTitle, "BOTTOM", 0, -5)
	GVAR.OptionsFrame.WarningSoundPullDown:SetHeight(18)
	TEMPLATE.EnablePullDownMenu(GVAR.OptionsFrame.WarningSoundPullDown)
	fw = fw + 5 + 18

	GVAR.OptionsFrame.WarningChannelSoundTitle = GVAR.OptionsFrame:CreateFontString(nil, "ARTWORK", "GameFontNormalSmall")
	GVAR.OptionsFrame.WarningChannelSoundTitle:SetPoint("LEFT", GVAR.OptionsFrame.WarningSoundPullDown, "RIGHT", 10, 0)
	GVAR.OptionsFrame.WarningChannelSoundTitle:SetJustifyH("LEFT")
	GVAR.OptionsFrame.WarningChannelSoundTitle:SetText(CHANNEL..": ")
	GVAR.OptionsFrame.WarningChannelSoundTitle:SetTextColor(1, 1, 1, 1)
	fw = fw + 0

	GVAR.OptionsFrame.WarningChannelSoundPullDown = CreateFrame("Button", nil, GVAR.OptionsFrame)
	TEMPLATE.PullDownMenu(
		GVAR.OptionsFrame.WarningChannelSoundPullDown,
		"WarningChannelSound",
		soundChannel[DrainSouler_Options.WarningChannelSound].name,
		0,
		#soundChannel,
		function(value)--WarningChannelSoundPullDownFunc
			DrainSouler_Options.WarningChannelSound = value
			if DrainSouler_Options.WarningSound > 1 then
				PlaySoundFile(soundsWarning[DrainSouler_Options.WarningSound].path, soundChannel[DrainSouler_Options.WarningChannelSound].channel)
			end
		end
	)
	GVAR.OptionsFrame.WarningChannelSoundPullDown:SetPoint("LEFT", GVAR.OptionsFrame.WarningChannelSoundTitle, "RIGHT", 5, 0)
	GVAR.OptionsFrame.WarningChannelSoundPullDown:SetHeight(18)
	TEMPLATE.EnablePullDownMenu(GVAR.OptionsFrame.WarningChannelSoundPullDown)
	fw = fw + 0

	-- - WarningHPSound
	GVAR.OptionsFrame.WarningHPSoundTitle = GVAR.OptionsFrame:CreateFontString(nil, "ARTWORK", "GameFontNormalSmall")
	GVAR.OptionsFrame.WarningHPSoundTitle:SetPoint("LEFT", GVAR.OptionsFrame, "LEFT", 20, 0)
	GVAR.OptionsFrame.WarningHPSoundTitle:SetPoint("TOP", GVAR.OptionsFrame.WarningSoundPullDown, "BOTTOM", 0, -10)
	GVAR.OptionsFrame.WarningHPSoundTitle:SetJustifyH("LEFT")
	GVAR.OptionsFrame.WarningHPSoundTitle:SetText(L["Warning Sound"].." - "..L["Target HP"]..": |cffffff99"..(DrainSouler_Options.WarningHPSound/1000).."K|r")
	GVAR.OptionsFrame.WarningHPSoundTitle:SetTextColor(1, 1, 1, 1)
	fw = fw + 10 + GVAR.OptionsFrame.WarningHPSoundTitle:GetStringHeight()

	GVAR.OptionsFrame.WarningHPSound = CreateFrame("Slider", nil, GVAR.OptionsFrame)
	TEMPLATE.Slider(
		GVAR.OptionsFrame.WarningHPSound,
		200,
		10000,
		0,
		1000000,
		DrainSouler_Options.WarningHPSound,
		function(self, value)--WarningHPSoundSliderFunc
			DrainSouler_Options.WarningHPSound = value
			GVAR.OptionsFrame.WarningHPSound:SetValue(DrainSouler_Options.WarningHPSound)
			GVAR.OptionsFrame.WarningHPSoundTitle:SetText(L["Warning Sound"].." - "..L["Target HP"]..": |cffffff99"..(DrainSouler_Options.WarningHPSound/1000).."K|r")
			GVAR.OptionsFrame.WarningSoundDescription:SetText( string.format(L["Plays a sound if your target is at or below %s health and the maximum health is greater than %s."], "|cffffff99"..DrainSouler_Options.WarningHPPSound.."%|r", "|cffffff99"..(DrainSouler_Options.WarningHPSound/1000).."K|r") )
		end,
		"K"
	)
	GVAR.OptionsFrame.WarningHPSound:SetPoint("TOPLEFT", GVAR.OptionsFrame.WarningHPSoundTitle, "BOTTOMLEFT", 0, -5)
	fw = fw + 5 + 17
	fw = fw + 1 + GVAR.OptionsFrame.WarningHPSound.textMin:GetStringHeight()

	GVAR.OptionsFrame.WarningHPPSoundTitle = GVAR.OptionsFrame:CreateFontString(nil, "ARTWORK", "GameFontNormalSmall")
	GVAR.OptionsFrame.WarningHPPSoundTitle:SetPoint("LEFT", GVAR.OptionsFrame.WarningHPSound, "RIGHT", 20, 0)
	GVAR.OptionsFrame.WarningHPPSoundTitle:SetPoint("TOP", GVAR.OptionsFrame.WarningHPSoundTitle, "TOP", 0, 0)
	GVAR.OptionsFrame.WarningHPPSoundTitle:SetJustifyH("LEFT")
	GVAR.OptionsFrame.WarningHPPSoundTitle:SetText(L["Warning Sound"].." - "..L["Target HP (%)"]..": |cffffff99"..DrainSouler_Options.WarningHPPSound.."%|r")
	GVAR.OptionsFrame.WarningHPPSoundTitle:SetTextColor(1, 1, 1, 1)
	fw = fw + 0

	GVAR.OptionsFrame.WarningHPPSound = CreateFrame("Slider", nil, GVAR.OptionsFrame)
	TEMPLATE.Slider(
		GVAR.OptionsFrame.WarningHPPSound,
		150,
		1,
		20,
		30,
		DrainSouler_Options.WarningHPPSound,
		function(self, value)--WarningHPPSoundSliderFunc
			DrainSouler_Options.WarningHPPSound = value
			GVAR.OptionsFrame.WarningHPPSound:SetValue(DrainSouler_Options.WarningHPPSound)
			GVAR.OptionsFrame.WarningHPPSoundTitle:SetText(L["Warning Sound"].." - "..L["Target HP (%)"]..": |cffffff99"..DrainSouler_Options.WarningHPPSound.."%|r")
			GVAR.OptionsFrame.WarningSoundDescription:SetText( string.format(L["Plays a sound if your target is at or below %s health and the maximum health is greater than %s."], "|cffffff99"..DrainSouler_Options.WarningHPPSound.."%|r", "|cffffff99"..(DrainSouler_Options.WarningHPSound/1000).."K|r") )
		end,
		"%"
	)
	GVAR.OptionsFrame.WarningHPPSound:SetPoint("TOPLEFT", GVAR.OptionsFrame.WarningHPPSoundTitle, "BOTTOMLEFT", 0, -5)
	fw = fw + 0

	GVAR.OptionsFrame.WarningSoundDescription = GVAR.OptionsFrame:CreateFontString(nil, "ARTWORK", "GameFontNormalSmall")
	GVAR.OptionsFrame.WarningSoundDescription:SetPoint("LEFT", GVAR.OptionsFrame, "LEFT", 20, 0)
	GVAR.OptionsFrame.WarningSoundDescription:SetPoint("TOP", GVAR.OptionsFrame.WarningHPPSound.textMin, "BOTTOM", 0, -5)
	GVAR.OptionsFrame.WarningSoundDescription:SetWidth(frameWidth-40)
	GVAR.OptionsFrame.WarningSoundDescription:SetJustifyH("LEFT")
	GVAR.OptionsFrame.WarningSoundDescription:SetTextColor(0.6, 0.6, 0.6, 1)
	GVAR.OptionsFrame.WarningSoundDescription:SetText( string.format(L["Plays a sound if your target is at or below %s health and the maximum health is greater than %s."], "|cffffff99"..DrainSouler_Options.WarningHPPSound.."%|r", "|cffffff99"..(DrainSouler_Options.WarningHPSound/1000).."K|r") )
	fw = fw + 5 + GVAR.OptionsFrame.WarningSoundDescription:GetStringHeight()

	GVAR.OptionsFrame.WarningSoundWarning = GVAR.OptionsFrame:CreateFontString(nil, "ARTWORK", "GameFontNormalSmall")
	GVAR.OptionsFrame.WarningSoundWarning:SetPoint("LEFT", GVAR.OptionsFrame, "LEFT", 20, 0)
	GVAR.OptionsFrame.WarningSoundWarning:SetPoint("TOP", GVAR.OptionsFrame.WarningSoundDescription, "BOTTOM", 0, -5)
	GVAR.OptionsFrame.WarningSoundWarning:SetJustifyH("LEFT")
	GVAR.OptionsFrame.WarningSoundWarning:SetTextColor(0.5, 0.5, 1, 1) -- WaRN
	GVAR.OptionsFrame.WarningSoundWarning:SetText(L["The general 'Target HP' and 'Target HP (%)' values are used first!"])
	fw = fw + 5 + GVAR.OptionsFrame.WarningSoundWarning:GetStringHeight()

	if DrainSouler_Options.WarningSound > 1 then
		TEMPLATE.EnableSlider(GVAR.OptionsFrame.WarningHPSound)
		TEMPLATE.EnableSlider(GVAR.OptionsFrame.WarningHPPSound)
		GVAR.OptionsFrame.WarningHPSoundTitle:SetTextColor(1, 1, 1, 1)
		GVAR.OptionsFrame.WarningHPPSoundTitle:SetTextColor(1, 1, 1, 1)
		GVAR.OptionsFrame.WarningSoundDescription:SetTextColor(0.6, 0.6, 0.6, 1)
		TEMPLATE.EnablePullDownMenu(GVAR.OptionsFrame.WarningChannelSoundPullDown)
		GVAR.OptionsFrame.WarningChannelSoundTitle:SetTextColor(1, 1, 1, 1)
	else
		TEMPLATE.DisableSlider(GVAR.OptionsFrame.WarningHPSound)
		TEMPLATE.DisableSlider(GVAR.OptionsFrame.WarningHPPSound)
		GVAR.OptionsFrame.WarningHPSoundTitle:SetTextColor(0.5, 0.5, 0.5, 1)
		GVAR.OptionsFrame.WarningHPPSoundTitle:SetTextColor(0.5, 0.5, 0.5, 1)
		GVAR.OptionsFrame.WarningSoundDescription:SetTextColor(0.5, 0.5, 0.5, 1)
		TEMPLATE.DisablePullDownMenu(GVAR.OptionsFrame.WarningChannelSoundPullDown)
		GVAR.OptionsFrame.WarningChannelSoundTitle:SetTextColor(0.5, 0.5, 0.5, 1)
	end

	-- Close
	GVAR.OptionsFrame.CloseConfig = CreateFrame("Button", nil, GVAR.OptionsFrame)
	TEMPLATE.TextButton(GVAR.OptionsFrame.CloseConfig, L["Close Configuration"], 1)
	GVAR.OptionsFrame.CloseConfig:SetPoint("BOTTOM", GVAR.OptionsFrame, "BOTTOM", 0, 10)
	GVAR.OptionsFrame.CloseConfig:SetWidth(frameWidth-20)
	GVAR.OptionsFrame.CloseConfig:SetHeight(30)
	GVAR.OptionsFrame.CloseConfig:SetScript("OnClick", function()
		DrainSouler_Options.configMode = false
		DrainSouler:DisableConfigMode()
	end)
	fw = fw + 10 + 30 + 10

	GVAR.OptionsFrame:SetClampRectInsets((frameWidth-50)/2, -((frameWidth-50)/2), -(fw-35), fw-35)
	GVAR.OptionsFrame:SetHeight(fw)

	-- Mover
	GVAR.OptionsFrame.MoverTop = CreateFrame("Frame", nil, GVAR.OptionsFrame)
	TEMPLATE.BorderTRBL(GVAR.OptionsFrame.MoverTop)
	GVAR.OptionsFrame.MoverTop:SetWidth(frameWidth)
	GVAR.OptionsFrame.MoverTop:SetHeight(20)
	GVAR.OptionsFrame.MoverTop:SetPoint("BOTTOM", GVAR.OptionsFrame, "TOP", 0, -1)
	GVAR.OptionsFrame.MoverTop:EnableMouse(true)
	GVAR.OptionsFrame.MoverTop:EnableMouseWheel(true)
	GVAR.OptionsFrame.MoverTop:SetScript("OnMouseWheel", NOOP)
	GVAR.OptionsFrame.MoverTopText = GVAR.OptionsFrame.MoverTop:CreateFontString(nil, "ARTWORK", "GameFontNormalSmall")
	GVAR.OptionsFrame.MoverTopText:SetPoint("CENTER", GVAR.OptionsFrame.MoverTop, "CENTER", 0, 0)
	GVAR.OptionsFrame.MoverTopText:SetJustifyH("CENTER")
	GVAR.OptionsFrame.MoverTopText:SetTextColor(0.3, 0.3, 0.3, 1)
	GVAR.OptionsFrame.MoverTopText:SetText(L["click & move"])

	GVAR.OptionsFrame.MoverBottom = CreateFrame("Frame", nil, GVAR.OptionsFrame)
	TEMPLATE.BorderTRBL(GVAR.OptionsFrame.MoverBottom)
	GVAR.OptionsFrame.MoverBottom:SetWidth(frameWidth)
	GVAR.OptionsFrame.MoverBottom:SetHeight(20)
	GVAR.OptionsFrame.MoverBottom:SetPoint("TOP", GVAR.OptionsFrame, "BOTTOM", 0, 1)
	GVAR.OptionsFrame.MoverBottom:EnableMouse(true)
	GVAR.OptionsFrame.MoverBottom:EnableMouseWheel(true)
	GVAR.OptionsFrame.MoverBottom:SetScript("OnMouseWheel", NOOP)
	GVAR.OptionsFrame.MoverBottomText = GVAR.OptionsFrame.MoverBottom:CreateFontString(nil, "ARTWORK", "GameFontNormalSmall")
	GVAR.OptionsFrame.MoverBottomText:SetPoint("CENTER", GVAR.OptionsFrame.MoverBottom, "CENTER", 0, 0)
	GVAR.OptionsFrame.MoverBottomText:SetJustifyH("CENTER")
	GVAR.OptionsFrame.MoverBottomText:SetTextColor(0.3, 0.3, 0.3, 1)
	GVAR.OptionsFrame.MoverBottomText:SetText(L["click & move"])

	GVAR.OptionsFrame.MoverTop:SetScript("OnEnter", function() GVAR.OptionsFrame.MoverTopText:SetTextColor(1, 1, 1, 1) end)
	GVAR.OptionsFrame.MoverTop:SetScript("OnLeave", function() GVAR.OptionsFrame.MoverTopText:SetTextColor(0.3, 0.3, 0.3, 1) end)
	GVAR.OptionsFrame.MoverTop:SetScript("OnMouseDown", function()
		GVAR.OptionsFrame:StartMoving()
	end)
	GVAR.OptionsFrame.MoverTop:SetScript("OnMouseUp", function()
		GVAR.OptionsFrame:StopMovingOrSizing()
		DrainSouler_Options.config_posX = GVAR.OptionsFrame:GetLeft()
		DrainSouler_Options.config_posY = GVAR.OptionsFrame:GetTop()
	end)

	GVAR.OptionsFrame.MoverBottom:SetScript("OnEnter", function() GVAR.OptionsFrame.MoverBottomText:SetTextColor(1, 1, 1, 1) end)
	GVAR.OptionsFrame.MoverBottom:SetScript("OnLeave", function() GVAR.OptionsFrame.MoverBottomText:SetTextColor(0.3, 0.3, 0.3, 1) end)
	GVAR.OptionsFrame.MoverBottom:SetScript("OnMouseDown", function()
		GVAR.OptionsFrame:StartMoving()
	end)
	GVAR.OptionsFrame.MoverBottom:SetScript("OnMouseUp", function()
		GVAR.OptionsFrame:StopMovingOrSizing()
		DrainSouler_Options.config_posX = GVAR.OptionsFrame:GetLeft()
		DrainSouler_Options.config_posY = GVAR.OptionsFrame:GetTop()
	end)
end
-- ---------------------------------------------------------------------------------------------------------------------

-- ---------------------------------------------------------------------------------------------------------------------
function DrainSouler:CreateMainFrame()
	local fontPath = _G["GameFontNormal"]:GetFont()

	GVAR.MainFrame = CreateFrame("Frame", nil, UIParent)
	GVAR.MainFrame:SetClampedToScreen(true)
	GVAR.MainFrame:SetClampRectInsets(0, 90, 0, -22)
	GVAR.MainFrame:SetMovable(true)
	GVAR.MainFrame:SetPoint("BOTTOM", GVAR.OptionsFrame, "TOP", -sizeTimerWidth+22, 50)
	GVAR.MainFrame:SetWidth(1)
	GVAR.MainFrame:SetHeight(1)
	GVAR.MainFrame:SetScale(1)
	GVAR.MainFrame:Hide()

	GVAR.MainFrame.WarningIcon = CreateFrame("Button", nil, GVAR.MainFrame)
	GVAR.MainFrame.WarningIcon:SetFrameLevel(GVAR.MainFrame:GetFrameLevel()+1)
	GVAR.MainFrame.WarningIcon:SetWidth(22)
	GVAR.MainFrame.WarningIcon:SetHeight(22)
	GVAR.MainFrame.WarningIcon:ClearAllPoints()
	GVAR.MainFrame.WarningIcon:SetPoint("TOPLEFT", GVAR.MainFrame, "TOPLEFT", 0, 0)
	GVAR.MainFrame.WarningIconBG = GVAR.MainFrame.WarningIcon:CreateTexture(nil, "BACKGROUND")
	GVAR.MainFrame.WarningIconBG:SetAllPoints(GVAR.MainFrame.WarningIcon)
	GVAR.MainFrame.WarningIconBG:SetTexture(0, 0, 0, 1)
	GVAR.MainFrame.WarningIconBG2 = GVAR.MainFrame.WarningIcon:CreateTexture(nil, "BORDER")
	GVAR.MainFrame.WarningIconBG2:SetWidth(20)
	GVAR.MainFrame.WarningIconBG2:SetHeight(20)
	GVAR.MainFrame.WarningIconBG2:SetPoint("TOPLEFT", GVAR.MainFrame.WarningIcon, "TOPLEFT", 1, -1)
	GVAR.MainFrame.WarningIconBG2:SetTexture(0.125, 0.262, 0.333, 0.6)
	GVAR.MainFrame.WarningIconTexture = GVAR.MainFrame.WarningIcon:CreateTexture(nil, "ARTWORK")
	GVAR.MainFrame.WarningIconTexture:SetWidth(18)
	GVAR.MainFrame.WarningIconTexture:SetHeight(18)
	GVAR.MainFrame.WarningIconTexture:SetPoint("TOPLEFT", GVAR.MainFrame.WarningIcon, "TOPLEFT", 2, -2)
	GVAR.MainFrame.WarningIconTexture:SetTexture(channelSpells[DrainSoulName].spellIcon)
	GVAR.MainFrame.WarningIconTexture:SetTexCoord(6/64, 58/64, 6/64, 58/64)

	GVAR.MainFrame.SpellIcon = CreateFrame("Button", nil, GVAR.MainFrame)
	GVAR.MainFrame.SpellIcon:SetFrameLevel(GVAR.MainFrame:GetFrameLevel()+1)
	GVAR.MainFrame.SpellIcon:SetWidth(16.5)
	GVAR.MainFrame.SpellIcon:SetHeight(16.5)
	GVAR.MainFrame.SpellIcon:ClearAllPoints()
	GVAR.MainFrame.SpellIcon:SetPoint("TOPLEFT", GVAR.MainFrame, "TOPLEFT", sizeTimerWidth+22, -(22-16.5))
	GVAR.MainFrame.SpellIcon:SetAlpha(0.75)
	GVAR.MainFrame.SpellIconBG = GVAR.MainFrame.SpellIcon:CreateTexture(nil, "BACKGROUND")
	GVAR.MainFrame.SpellIconBG:SetAllPoints(GVAR.MainFrame.SpellIcon)
	GVAR.MainFrame.SpellIconBG:SetTexture(0, 0, 0, 1)
	GVAR.MainFrame.SpellIconTexture = GVAR.MainFrame.SpellIcon:CreateTexture(nil, "ARTWORK")
	GVAR.MainFrame.SpellIconTexture:SetWidth(14.5)--9)
	GVAR.MainFrame.SpellIconTexture:SetHeight(14.5)--9)
	GVAR.MainFrame.SpellIconTexture:SetPoint("TOPLEFT", GVAR.MainFrame.SpellIcon, "TOPLEFT", 1, -1)
	GVAR.MainFrame.SpellIconTexture:SetTexture(channelSpells[DrainSoulName].spellIcon)
	GVAR.MainFrame.SpellIconTexture:SetTexCoord(6/64, 58/64, 6/64, 58/64)

	GVAR.MainFrame.HP = CreateFrame("Button", nil, GVAR.MainFrame)
	GVAR.MainFrame.HP:SetFrameLevel(GVAR.MainFrame:GetFrameLevel()+1)
	GVAR.MainFrame.HP:SetWidth(sizeTimerWidth)
	GVAR.MainFrame.HP:SetHeight(11)
	GVAR.MainFrame.HP:ClearAllPoints()
	GVAR.MainFrame.HP:SetPoint("TOPLEFT", GVAR.MainFrame, "TOPLEFT", 22, 0)
	GVAR.MainFrame.HPBG = GVAR.MainFrame.HP:CreateTexture(nil, "BACKGROUND")
	GVAR.MainFrame.HPBG:SetAllPoints(GVAR.MainFrame.HP)
	GVAR.MainFrame.HPBG:SetTexture(0, 0, 0, 1)
	GVAR.MainFrame.HPText = GVAR.MainFrame.HP:CreateFontString(nil, "ARTWORK", "GameFontNormal")
	GVAR.MainFrame.HPText:SetFont(fontPath, 11, "")
	GVAR.MainFrame.HPText:SetAllPoints(GVAR.MainFrame.HP)
	GVAR.MainFrame.HPText:SetJustifyH("CENTER")
	GVAR.MainFrame.HPText:SetJustifyV("MIDDLE")
	GVAR.MainFrame.HPText:SetTextColor(1, 1, 1, 1)

	GVAR.MainFrame.DMG = CreateFrame("Button", nil, GVAR.MainFrame)
	GVAR.MainFrame.DMG:SetFrameLevel(GVAR.MainFrame:GetFrameLevel()+1)
	GVAR.MainFrame.DMG:SetWidth(sizeTimerWidth)
	GVAR.MainFrame.DMG:SetHeight(11)
	GVAR.MainFrame.DMG:ClearAllPoints()
	GVAR.MainFrame.DMG:SetPoint("TOPLEFT", GVAR.MainFrame, "TOPLEFT", 22, -11)
	GVAR.MainFrame.DMGBG = GVAR.MainFrame.DMG:CreateTexture(nil, "BACKGROUND")
	GVAR.MainFrame.DMGBG:SetAllPoints(GVAR.MainFrame.DMG)
	GVAR.MainFrame.DMGBG:SetTexture(0, 0, 0, 1)
	GVAR.MainFrame.DMGText = GVAR.MainFrame.DMG:CreateFontString(nil, "ARTWORK", "GameFontNormal")
	GVAR.MainFrame.DMGText:SetFont(fontPath, 11, "")
	GVAR.MainFrame.DMGText:SetAllPoints(GVAR.MainFrame.DMG)
	GVAR.MainFrame.DMGText:SetJustifyH("CENTER")
	GVAR.MainFrame.DMGText:SetJustifyV("MIDDLE")
	GVAR.MainFrame.DMGText:SetTextColor(1, 1, 1, 1)

	GVAR.MainFrame.TickTimer = CreateFrame("Button", nil, GVAR.MainFrame)
	GVAR.MainFrame.TickTimer:SetFrameLevel(GVAR.MainFrame:GetFrameLevel()+1)
	GVAR.MainFrame.TickTimer:SetWidth(sizeTimerWidth)
	GVAR.MainFrame.TickTimer:SetHeight(11)
	GVAR.MainFrame.TickTimer:ClearAllPoints()
	GVAR.MainFrame.TickTimer:SetPoint("TOPLEFT", GVAR.MainFrame, "TOPLEFT", 22, -11)
	GVAR.MainFrame.TickTimerTexture = GVAR.MainFrame.TickTimer:CreateTexture(nil, "ARTWORK")
	GVAR.MainFrame.TickTimerTexture:SetWidth(0.01)
	GVAR.MainFrame.TickTimerTexture:SetHeight(6)
	GVAR.MainFrame.TickTimerTexture:SetPoint("TOPLEFT", GVAR.MainFrame, "TOPLEFT", 22, -13.5)
	GVAR.MainFrame.TickTimerTexture:SetTexture(channelSpells[DrainSoulName].timerColor[1], channelSpells[DrainSoulName].timerColor[2], channelSpells[DrainSoulName].timerColor[3], 1)

	GVAR.MainFrame.ChannelTimer = CreateFrame("Button", nil, GVAR.MainFrame)
	GVAR.MainFrame.ChannelTimer:SetFrameLevel(GVAR.MainFrame:GetFrameLevel()+1)
	GVAR.MainFrame.ChannelTimer:SetWidth(sizeTimerWidth)
	GVAR.MainFrame.ChannelTimer:SetHeight(2)
	GVAR.MainFrame.ChannelTimer:ClearAllPoints()
	GVAR.MainFrame.ChannelTimer:SetPoint("TOPLEFT", GVAR.MainFrame, "TOPLEFT", 22, -22)
	GVAR.MainFrame.ChannelTimer:EnableMouse(false)
	GVAR.MainFrame.ChannelTimerTexture = GVAR.MainFrame.ChannelTimer:CreateTexture(nil, "ARTWORK")
	GVAR.MainFrame.ChannelTimerTexture:SetWidth(0.01)
	GVAR.MainFrame.ChannelTimerTexture:SetHeight(2)
	GVAR.MainFrame.ChannelTimerTexture:SetPoint("TOPLEFT", GVAR.MainFrame, "TOPLEFT", 22, -22)
	GVAR.MainFrame.ChannelTimerTexture:SetTexture(0.5, 0.5, 1, 1)

	GVAR.MainFrame.Tick = CreateFrame("Button", nil, GVAR.MainFrame)
	GVAR.MainFrame.Tick:SetFrameLevel(GVAR.MainFrame:GetFrameLevel()+2)
	GVAR.MainFrame.Tick:SetWidth(22)
	GVAR.MainFrame.Tick:SetHeight(22)
	GVAR.MainFrame.Tick:ClearAllPoints()
	GVAR.MainFrame.Tick:SetPoint("TOPLEFT", GVAR.MainFrame, "TOPLEFT", 22, 0)
	GVAR.MainFrame.TickBG = GVAR.MainFrame.Tick:CreateTexture(nil, "BACKGROUND")
	GVAR.MainFrame.TickBG:SetAllPoints(GVAR.MainFrame.Tick)
	GVAR.MainFrame.TickBG:SetTexture(1, 1, 1, 0)
	GVAR.MainFrame.TickText = GVAR.MainFrame.Tick:CreateFontString(nil, "ARTWORK", "GameFontNormal")
	GVAR.MainFrame.TickText:SetFont(fontPath, 20, "")
	GVAR.MainFrame.TickText:SetPoint("LEFT", GVAR.MainFrame.Tick, "LEFT", 0, 0)
	GVAR.MainFrame.TickText:SetJustifyH("LEFT")
	GVAR.MainFrame.TickText:SetJustifyV("MIDDLE")
	GVAR.MainFrame.TickText:SetTextColor(1, 1, 1, 1)

	GVAR.MainFrame.Duration = CreateFrame("Button", nil, GVAR.MainFrame)
	GVAR.MainFrame.Duration:SetFrameLevel(GVAR.MainFrame:GetFrameLevel()+1)
	GVAR.MainFrame.Duration:SetWidth(sizeTimerWidth*4)
	GVAR.MainFrame.Duration:SetHeight(10)
	GVAR.MainFrame.Duration:ClearAllPoints()
	GVAR.MainFrame.Duration:SetPoint("TOPLEFT", GVAR.MainFrame, "TOPLEFT", 22, -24)
	GVAR.MainFrame.Duration:EnableMouse(false)
	GVAR.MainFrame.DurationText = GVAR.MainFrame.Duration:CreateFontString(nil, "ARTWORK", "GameFontNormal")
	GVAR.MainFrame.DurationText:SetFont(fontPath, 8, "")
	GVAR.MainFrame.DurationText:SetAllPoints(GVAR.MainFrame.Duration)
	GVAR.MainFrame.DurationText:SetJustifyH("LEFT")
	GVAR.MainFrame.DurationText:SetJustifyV("MIDDLE")
	GVAR.MainFrame.DurationText:SetTextColor(0.8, 0.8, 0.8, 1)

	GVAR.MainFrame.Modifiers = CreateFrame("Button", nil, GVAR.MainFrame)
	GVAR.MainFrame.Modifiers:SetFrameLevel(GVAR.MainFrame:GetFrameLevel()+1)
	GVAR.MainFrame.Modifiers:SetWidth(sizeTimerWidth*4)
	GVAR.MainFrame.Modifiers:SetHeight(10)
	GVAR.MainFrame.Modifiers:ClearAllPoints()
	GVAR.MainFrame.Modifiers:SetPoint("TOPLEFT", GVAR.MainFrame.Duration, "BOTTOMLEFT", 0, 0)
	GVAR.MainFrame.Modifiers:EnableMouse(false)
	GVAR.MainFrame.ModifiersText = GVAR.MainFrame.Modifiers:CreateFontString(nil, "ARTWORK", "GameFontNormal")
	GVAR.MainFrame.ModifiersText:SetFont(fontPath, 8, "")
	GVAR.MainFrame.ModifiersText:SetAllPoints(GVAR.MainFrame.Modifiers)
	GVAR.MainFrame.ModifiersText:SetJustifyH("LEFT")
	GVAR.MainFrame.ModifiersText:SetJustifyV("MIDDLE")
	GVAR.MainFrame.ModifiersText:SetTextColor(0.8, 0.8, 0.8, 1)
end
-- ---------------------------------------------------------------------------------------------------------------------

-- ---------------------------------------------------------------------------------------------------------------------
-- -------------------------------------------------------------------------- --
function DrainSouler:DisableMovement()
	GVAR.MainFrame.WarningIcon:SetScript("OnMouseDown", nil)
	GVAR.MainFrame.WarningIcon:SetScript("OnMouseUp", nil)
	GVAR.MainFrame.WarningIcon:SetScript("OnHide", nil)
	GVAR.MainFrame.WarningIcon:EnableMouse(false)
	GVAR.MainFrame.SpellIcon:SetScript("OnMouseDown", nil)
	GVAR.MainFrame.SpellIcon:SetScript("OnMouseUp", nil)
	GVAR.MainFrame.SpellIcon:SetScript("OnHide", nil)
	GVAR.MainFrame.SpellIcon:EnableMouse(false)
	GVAR.MainFrame.HP:SetScript("OnMouseDown", nil)
	GVAR.MainFrame.HP:SetScript("OnMouseUp", nil)
	GVAR.MainFrame.HP:SetScript("OnHide", nil)
	GVAR.MainFrame.HP:EnableMouse(false)
	GVAR.MainFrame.DMG:SetScript("OnMouseDown", nil)
	GVAR.MainFrame.DMG:SetScript("OnMouseUp", nil)
	GVAR.MainFrame.DMG:SetScript("OnHide", nil)
	GVAR.MainFrame.DMG:EnableMouse(false)
	GVAR.MainFrame.TickTimer:SetScript("OnMouseDown", nil)
	GVAR.MainFrame.TickTimer:SetScript("OnMouseUp", nil)
	GVAR.MainFrame.TickTimer:SetScript("OnHide", nil)
	GVAR.MainFrame.TickTimer:EnableMouse(false)
	GVAR.MainFrame.Tick:SetScript("OnMouseDown", nil)
	GVAR.MainFrame.Tick:SetScript("OnMouseUp", nil)
	GVAR.MainFrame.Tick:SetScript("OnHide", nil)
	GVAR.MainFrame.Tick:EnableMouse(false)
end
-- -------------------------------------------------------------------------- --

-- -------------------------------------------------------------------------- --
local function OnMouseDown()
	GVAR.MainFrame:StartMoving()
end

local function OnMouseUp()
	GVAR.MainFrame:StopMovingOrSizing()
	DrainSouler_Options.posX = GVAR.MainFrame:GetLeft()
	DrainSouler_Options.posY = GVAR.MainFrame:GetTop()
end

function DrainSouler:EnableMovement()
	GVAR.MainFrame.WarningIcon:SetScript("OnMouseDown", OnMouseDown)
	GVAR.MainFrame.WarningIcon:SetScript("OnMouseUp", OnMouseUp)
	GVAR.MainFrame.WarningIcon:SetScript("OnHide", OnMouseUp)
	GVAR.MainFrame.WarningIcon:EnableMouse(true)
	GVAR.MainFrame.SpellIcon:SetScript("OnMouseDown", OnMouseDown)
	GVAR.MainFrame.SpellIcon:SetScript("OnMouseUp", OnMouseUp)
	GVAR.MainFrame.SpellIcon:SetScript("OnHide", OnMouseUp)
	GVAR.MainFrame.SpellIcon:EnableMouse(true)
	GVAR.MainFrame.HP:SetScript("OnMouseDown", OnMouseDown)
	GVAR.MainFrame.HP:SetScript("OnMouseUp", OnMouseUp)
	GVAR.MainFrame.HP:SetScript("OnHide", OnMouseUp)
	GVAR.MainFrame.HP:EnableMouse(true)
	GVAR.MainFrame.DMG:SetScript("OnMouseDown", OnMouseDown)
	GVAR.MainFrame.DMG:SetScript("OnMouseUp", OnMouseUp)
	GVAR.MainFrame.DMG:SetScript("OnHide", OnMouseUp)
	GVAR.MainFrame.DMG:EnableMouse(true)
	GVAR.MainFrame.TickTimer:SetScript("OnMouseDown", OnMouseDown)
	GVAR.MainFrame.TickTimer:SetScript("OnMouseUp", OnMouseUp)
	GVAR.MainFrame.TickTimer:SetScript("OnHide", OnMouseUp)
	GVAR.MainFrame.TickTimer:EnableMouse(true)
	GVAR.MainFrame.Tick:SetScript("OnMouseDown", OnMouseDown)
	GVAR.MainFrame.Tick:SetScript("OnMouseUp", OnMouseUp)
	GVAR.MainFrame.Tick:SetScript("OnHide", OnMouseUp)
	GVAR.MainFrame.Tick:EnableMouse(true)
end
-- -------------------------------------------------------------------------- --

-- -------------------------------------------------------------------------- --
function DrainSouler:SetupSize()
	GVAR.MainFrame.WarningIcon:SetScale(DrainSouler_Options.Size)
	GVAR.MainFrame.SpellIcon:SetScale(DrainSouler_Options.Size)
	GVAR.MainFrame.HP:SetScale(DrainSouler_Options.Size)
	GVAR.MainFrame.DMG:SetScale(DrainSouler_Options.Size)
	GVAR.MainFrame.TickTimer:SetScale(DrainSouler_Options.Size)
	GVAR.MainFrame.ChannelTimer:SetScale(DrainSouler_Options.Size)
	GVAR.MainFrame.Tick:SetScale(DrainSouler_Options.Size)
	GVAR.MainFrame.Duration:SetScale(DrainSouler_Options.Size)
	GVAR.MainFrame.Modifiers:SetScale(DrainSouler_Options.Size)
end
-- -------------------------------------------------------------------------- --

-- -------------------------------------------------------------------------- --
function DrainSouler:EnableConfigMode()
	configMode = true
	GVAR.MainFrame.TickTimer:SetScript("OnUpdate", nil)
	GVAR.MainFrame.TickTimer:Hide()
	GVAR.MainFrame.ChannelTimer:Hide()
	if ((DrainSouler_Options.TalentDependent and isAffliction) or (not DrainSouler_Options.TalentDependent)) then
		GVAR.MainFrame:Show()
		GVAR.MainFrame.WarningIcon:Show()
	else
		GVAR.MainFrame:Hide()
	end
	GVAR.OptionsFrame:Show()
	
	GVAR.MainFrame.HPText:SetText(L["HP%"])
	GVAR.MainFrame.DMGText:SetText(L["DMG"])
	GVAR.MainFrame.TickText:SetText(L["X"])
	if DrainSouler_Options.DetailInfo then
		GVAR.MainFrame.DurationText:SetText("numTicks | tick duration in ms | complete duration in ms")
		GVAR.MainFrame.ModifiersText:SetText(STAT_HASTE.." |cffffff82+x%|r | "..STAT_SPELLPOWER.." |cffffff82+x%|r")
	else
		GVAR.MainFrame.DurationText:SetText("")
		GVAR.MainFrame.ModifiersText:SetText("")
	end

	GVAR.MainFrame.SpellIconTexture:SetTexture(channelSpells[DrainSoulName].spellIcon)
	GVAR.MainFrame.SpellIcon:Show()

	DrainSouler:DisableAllEvents()
end
-- -------------------------------------------------------------------------- --

-- -------------------------------------------------------------------------- --
function DrainSouler:DisableConfigMode()
	configMode = nil
	GVAR.OptionsFrame:Hide()

	GVAR.MainFrame.HPText:SetText("")
	GVAR.MainFrame.DMGText:SetText("")
	GVAR.MainFrame.TickText:SetText("")
	GVAR.MainFrame.DurationText:SetText("")
	GVAR.MainFrame.ModifiersText:SetText("")

	GVAR.MainFrame.SpellIcon:Hide()

	if InCombatLockdown() then
		inCombat = true
	else
		inCombat = false
	end

	if ((DrainSouler_Options.TalentDependent and isAffliction) or (not DrainSouler_Options.TalentDependent)) then
		DrainSouler:EnableAllEvents()
		DrainSouler:CheckTarget()
	end
end
-- -------------------------------------------------------------------------- --
-- ---------------------------------------------------------------------------------------------------------------------



-- ---------------------------------------------------------------------------------------------------------------------
function DrainSouler:Initialize()
	if DrainSouler_Options.Lock then
		DrainSouler:DisableMovement()
		GVAR.OptionsFrame.MoveNote:SetTextColor(0, 0, 0, 1)
	else
		DrainSouler:EnableMovement()
		GVAR.OptionsFrame.MoveNote:SetTextColor(1, 1, 0.5, 1)
	end

	DrainSouler:SetupSize()

	if DrainSouler_Options.posX then
		GVAR.MainFrame:SetPoint("TOPLEFT", UIParent, "BOTTOMLEFT", DrainSouler_Options.posX, DrainSouler_Options.posY)
	end

	if DrainSouler_Options.config_posX then
		GVAR.OptionsFrame:SetPoint("TOPLEFT", UIParent, "BOTTOMLEFT", DrainSouler_Options.config_posX, DrainSouler_Options.config_posY)
	end

	if DrainSouler_Options.configMode then
		DrainSouler:EnableConfigMode()
	else
		DrainSouler:DisableConfigMode()
	end
end
-- ---------------------------------------------------------------------------------------------------------------------



-- ---------------------------------------------------------------------------------------------------------------------
-- -------------------------------------------------------------------------- --
function DrainSouler:CheckTalent()
	isAffliction = nil
	currentSpecName = nil
	local primaryTalentTree = GetSpecialization()
	if primaryTalentTree then
		local _, specName = GetSpecializationInfo(primaryTalentTree)
		if specName and specName ~= "" then
			currentSpecName = specName
			if specName == afflictionSpecName then
				isAffliction = true
			end
		end
	end
	DrainSouler:UpdateTalentDisplay()
end
-- -------------------------------------------------------------------------- --

-- -------------------------------------------------------------------------- --
function DrainSouler:UpdateTalentDisplay()
	if not GVAR.MainFrame then return end
	if not GVAR.OptionsFrame then return end

	GVAR.OptionsFrame.TalentDependentDescription:SetText(L["Your current Spec:"].." |cffffff99"..(currentSpecName or UNKNOWN).."|r")

	if ((DrainSouler_Options.TalentDependent and isAffliction) or (not DrainSouler_Options.TalentDependent)) then
		GVAR.MainFrame:Show()
		GVAR.MainFrame.WarningIcon:Show()
		-- MoveNote
		GVAR.OptionsFrame.MoveNote:SetTextColor(1, 1, 0.5, 1)
		-- Lock
		TEMPLATE.EnableCheckButton(GVAR.OptionsFrame.Lock)
		-- onlyInCombat
		TEMPLATE.EnableCheckButton(GVAR.OptionsFrame.onlyInCombat)
		-- DetailInfo
		TEMPLATE.EnableCheckButton(GVAR.OptionsFrame.DetailInfo)
		-- Size
		GVAR.OptionsFrame.SizeTitle:SetTextColor(1, 1, 1, 1)
		TEMPLATE.EnableSlider(GVAR.OptionsFrame.Size)
		-- minHP
		GVAR.OptionsFrame.minHPTitle:SetTextColor(1, 1, 1, 1)
		TEMPLATE.EnableSlider(GVAR.OptionsFrame.minHP)
		GVAR.OptionsFrame.minHPDescription:SetTextColor(0.6, 0.6, 0.6, 1)
		-- minHPP
		GVAR.OptionsFrame.minHPPTitle:SetTextColor(1, 1, 1, 1)
		TEMPLATE.EnableSlider(GVAR.OptionsFrame.minHPP)
		-- TickSound
		GVAR.OptionsFrame.TickSoundTitle:SetTextColor(1, 1, 1, 1)
		for i = 1, #channelINTspells do
			Desaturation(GVAR.OptionsFrame.TickSoundImg[i].BG, false)
			Desaturation(GVAR.OptionsFrame.TickSoundImg[i].Texture, false)
			TEMPLATE.EnablePullDownMenu(GVAR.OptionsFrame.TickSoundPullDown[i])
			if DrainSouler_Options.TickSound[i] > 1 then
				TEMPLATE.EnablePullDownMenu(GVAR.OptionsFrame.TickChannelPullDown[i])
				GVAR.OptionsFrame.TickChannelTitle[i]:SetTextColor(1, 1, 1, 1)
			else
				TEMPLATE.DisablePullDownMenu(GVAR.OptionsFrame.TickChannelPullDown[i])
				GVAR.OptionsFrame.TickChannelTitle[i]:SetTextColor(0.5, 0.5, 0.5, 1)
			end
		end
		GVAR.OptionsFrame.TickSoundDescription:SetTextColor(0.6, 0.6, 0.6, 1)

		-- WarningSound
		GVAR.OptionsFrame.WarningSoundTitle:SetTextColor(1, 1, 1, 1)
		TEMPLATE.EnablePullDownMenu(GVAR.OptionsFrame.WarningSoundPullDown)

		if DrainSouler_Options.WarningSound > 1 then
			TEMPLATE.EnableSlider(GVAR.OptionsFrame.WarningHPSound)
			TEMPLATE.EnableSlider(GVAR.OptionsFrame.WarningHPPSound)
			GVAR.OptionsFrame.WarningHPSoundTitle:SetTextColor(1, 1, 1, 1)
			GVAR.OptionsFrame.WarningHPPSoundTitle:SetTextColor(1, 1, 1, 1)
			GVAR.OptionsFrame.WarningSoundDescription:SetTextColor(0.6, 0.6, 0.6, 1)
			GVAR.OptionsFrame.WarningSoundWarning:SetTextColor(0.5, 0.5, 1, 1) -- WaRN
			TEMPLATE.EnablePullDownMenu(GVAR.OptionsFrame.WarningChannelSoundPullDown)
			GVAR.OptionsFrame.WarningChannelSoundTitle:SetTextColor(1, 1, 1, 1)
		else
			TEMPLATE.DisableSlider(GVAR.OptionsFrame.WarningHPSound)
			TEMPLATE.DisableSlider(GVAR.OptionsFrame.WarningHPPSound)
			GVAR.OptionsFrame.WarningHPSoundTitle:SetTextColor(0.5, 0.5, 0.5, 1)
			GVAR.OptionsFrame.WarningHPPSoundTitle:SetTextColor(0.5, 0.5, 0.5, 1)
			GVAR.OptionsFrame.WarningSoundDescription:SetTextColor(0.5, 0.5, 0.5, 1)
			GVAR.OptionsFrame.WarningSoundWarning:SetTextColor(0.5, 0.5, 0.5, 1)
			TEMPLATE.DisablePullDownMenu(GVAR.OptionsFrame.WarningChannelSoundPullDown)
			GVAR.OptionsFrame.WarningChannelSoundTitle:SetTextColor(0.5, 0.5, 0.5, 1)
		end

		if not DrainSouler_Options.Lock then
			GVAR.OptionsFrame.MoveNote:SetTextColor(1, 1, 0.5, 1)
		else
			GVAR.OptionsFrame.MoveNote:SetTextColor(0, 0, 0, 1)
		end

		if not configMode then
			if InCombatLockdown() then
				inCombat = true
			else
				inCombat = false
			end
			DrainSouler:EnableAllEvents()
			DrainSouler:CheckTarget()
		end
	else
		GVAR.MainFrame:Hide()
		-- MoveNote
		GVAR.OptionsFrame.MoveNote:SetTextColor(0.5, 0.5, 0.5, 1)
		-- Lock
		TEMPLATE.DisableCheckButton(GVAR.OptionsFrame.Lock)
		-- onlyInCombat
		TEMPLATE.DisableCheckButton(GVAR.OptionsFrame.onlyInCombat)
		-- DetailInfo
		TEMPLATE.DisableCheckButton(GVAR.OptionsFrame.DetailInfo)
		-- Size
		GVAR.OptionsFrame.SizeTitle:SetTextColor(0.5, 0.5, 0.5, 1)
		TEMPLATE.DisableSlider(GVAR.OptionsFrame.Size)
		-- minHP
		GVAR.OptionsFrame.minHPTitle:SetTextColor(0.5, 0.5, 0.5, 1)
		TEMPLATE.DisableSlider(GVAR.OptionsFrame.minHP)
		GVAR.OptionsFrame.minHPDescription:SetTextColor(0.5, 0.5, 0.5, 1)
		-- minHPP
		GVAR.OptionsFrame.minHPPTitle:SetTextColor(0.5, 0.5, 0.5, 1)
		TEMPLATE.DisableSlider(GVAR.OptionsFrame.minHPP)
		-- TickSound
		GVAR.OptionsFrame.TickSoundTitle:SetTextColor(0.5, 0.5, 0.5, 1)
		for i = 1, #channelINTspells do
			Desaturation(GVAR.OptionsFrame.TickSoundImg[i].BG, true)
			Desaturation(GVAR.OptionsFrame.TickSoundImg[i].Texture, true)
			TEMPLATE.DisablePullDownMenu(GVAR.OptionsFrame.TickSoundPullDown[i])
			TEMPLATE.DisablePullDownMenu(GVAR.OptionsFrame.TickChannelPullDown[i])
			GVAR.OptionsFrame.TickChannelTitle[i]:SetTextColor(0.5, 0.5, 0.5, 1)
		end
		GVAR.OptionsFrame.TickSoundDescription:SetTextColor(0.5, 0.5, 0.5, 1)
		-- WarningSound
		GVAR.OptionsFrame.WarningSoundTitle:SetTextColor(0.5, 0.5, 0.5, 1)
		TEMPLATE.DisablePullDownMenu(GVAR.OptionsFrame.WarningSoundPullDown)
		GVAR.OptionsFrame.WarningSoundDescription:SetTextColor(0.5, 0.5, 0.5, 1)
		TEMPLATE.DisableSlider(GVAR.OptionsFrame.WarningHPSound)
		TEMPLATE.DisableSlider(GVAR.OptionsFrame.WarningHPPSound)
		GVAR.OptionsFrame.WarningHPSoundTitle:SetTextColor(0.5, 0.5, 0.5, 1)
		GVAR.OptionsFrame.WarningHPPSoundTitle:SetTextColor(0.5, 0.5, 0.5, 1)
		GVAR.OptionsFrame.WarningSoundWarning:SetTextColor(0.5, 0.5, 0.5, 1)
		TEMPLATE.DisablePullDownMenu(GVAR.OptionsFrame.WarningChannelSoundPullDown)
		GVAR.OptionsFrame.WarningChannelSoundTitle:SetTextColor(0.5, 0.5, 0.5, 1)

		GVAR.OptionsFrame.MoveNote:SetTextColor(0, 0, 0, 1)

		if not configMode then
			DrainSouler:DisableAllEvents()
		end
	end
end
-- -------------------------------------------------------------------------- --
-- ---------------------------------------------------------------------------------------------------------------------



-- ---------------------------------------------------------------------------------------------------------------------
-- -------------------------------------------------------------------------- --
function DrainSouler:CheckHealth()
	local DrainSouler_Options = DrainSouler_Options

	local maxHealth = UnitHealthMax("target")
	if maxHealth < DrainSouler_Options.minHP then
		GVAR.MainFrame:Hide()
		return
	end

	local curHealthP = UnitHealth("target") / maxHealth * 100
	if curHealthP > 0 and curHealthP <= DrainSouler_Options.minHPP then
		GVAR.MainFrame:Show()
		GVAR.MainFrame.HPText:SetText( math_floor(curHealthP * mult2 + 0.5) / mult2.."%" )

		if DrainSouler_Options.WarningSound > 1 then
			if curHealthP <= DrainSouler_Options.WarningHPPSound and maxHealth > DrainSouler_Options.WarningHPSound then
				local mobGUID = UnitGUID("target")
				if not mobTable[mobGUID] then
					PlaySoundFile(soundsWarning[DrainSouler_Options.WarningSound].path, soundChannel[DrainSouler_Options.WarningChannelSound].channel)
					mobTable[mobGUID] = GetTime()
				end
			end
		end

		if curHealthP <= 20 then
			GVAR.MainFrame.WarningIcon:Show()
		else
			GVAR.MainFrame.WarningIcon:Hide()
		end
	else
		GVAR.MainFrame:Hide()
		GVAR.MainFrame.HPText:SetText("")
		GVAR.MainFrame.DMGText:SetText("")
		GVAR.MainFrame.TickText:SetText("")
	end
	if not channeling then
		GVAR.MainFrame.TickTimer:Hide()
	end
end
-- -------------------------------------------------------------------------- --

-- -------------------------------------------------------------------------- --
function DrainSouler:DisableAllEvents(TargetCheck)
	DrainSouler:UnregisterEvent("PLAYER_TARGET_CHANGED")
	DrainSouler:UnregisterEvent("UNIT_SPELLCAST_SUCCEEDED")
	DrainSouler:UnregisterEvent("UNIT_SPELLCAST_CHANNEL_UPDATE")
	DrainSouler:UnregisterEvent("UNIT_SPELLCAST_CHANNEL_STOP")
	DrainSouler:UnregisterEvent("PLAYER_REGEN_DISABLED")
	DrainSouler:UnregisterEvent("PLAYER_REGEN_ENABLED")
	DrainSouler:UnregisterEvent("UNIT_HEALTH")
	DrainSouler:UnregisterEvent("PLAYER_DAMAGE_DONE_MODS")
	DrainSouler:UnregisterEvent("COMBAT_LOG_EVENT_UNFILTERED")
end

function DrainSouler:EnableAllEvents()
	DrainSouler:RegisterEvent("PLAYER_TARGET_CHANGED")
	DrainSouler:RegisterUnitEvent("UNIT_SPELLCAST_SUCCEEDED", "player")
	DrainSouler:RegisterUnitEvent("UNIT_SPELLCAST_CHANNEL_UPDATE", "player")
	DrainSouler:RegisterUnitEvent("UNIT_SPELLCAST_CHANNEL_STOP", "player")
	DrainSouler:RegisterEvent("PLAYER_REGEN_DISABLED")
	DrainSouler:RegisterEvent("PLAYER_REGEN_ENABLED")
	if DrainSouler_Options.DetailInfo then
		DrainSouler:RegisterEvent("PLAYER_DAMAGE_DONE_MODS")
	end
end
-- -------------------------------------------------------------------------- --

-- -------------------------------------------------------------------------- --
function DrainSouler:CheckTarget()
	if (not inCombat and DrainSouler_Options.onlyInCombat) or UnitInVehicle("player") or UnitHasVehicleUI("player") then
		GVAR.MainFrame:Hide()
		DrainSouler:UnregisterEvent("UNIT_HEALTH")
		return
	end

	if UnitExists("target") then
		DrainSouler:CheckHealth()
		DrainSouler:RegisterUnitEvent("UNIT_HEALTH", "target")
	else
		GVAR.MainFrame:Hide()
		DrainSouler:UnregisterEvent("UNIT_HEALTH")
	end
end
-- -------------------------------------------------------------------------- --

-- -------------------------------------------------------------------------- --
function DrainSouler:UpdateDetailedInfo()
	if channeling then
		local modHaste = math_floor(startHaste * mult2 + 0.5) / mult2
		local curHaste = UnitSpellHaste("player")
		if curHaste > startHaste then
			if curHaste > 0 and startHaste > 0 then
				local dif = ((curHaste / startHaste)-1)*100
				modHaste = modHaste.." |cffffff82+"..(math_floor(dif * mult2 + 0.5) / mult2).."%|r"
			else
				modHaste = modHaste.." |cffffff82+x%|r"
			end
		end
		local modDamage = startDamage
		local curDamage = GetSpellBonusDamage(6)
		if curDamage > startDamage then
			if curDamage > 0 and startDamage > 0 then
				local dif = ((curDamage / startDamage)-1)*100
				modDamage = modDamage.." |cffffff82"..(math_floor(dif * mult2 + 0.5) / mult2).."%|r"
			else
				modDamage = modDamage.." |cffffff82+x%|r"
			end
		end
		GVAR.MainFrame.ModifiersText:SetText(modHaste.." | "..modDamage)
	else
		local curHaste = math_floor(UnitSpellHaste("player") * mult2 + 0.5) / mult2
		local curDamage = GetSpellBonusDamage(6)
		GVAR.MainFrame.ModifiersText:SetText(curHaste.." | "..curDamage)
	end
end
-- -------------------------------------------------------------------------- --

-- -------------------------------------------------------------------------- --
local throttle = 0.05
local elapsed = 0
TickUpdate = function(self, elap)
	elapsed = elapsed + elap
	if elapsed < throttle then return end
	elapsed = 0

	local curTime1000 = GetTime()*1000

	-- tick timer
	local width = ((curTickStart + curTickDura - curTime1000) / curTickDura) * sizeTimerWidth
	width = math_max(0.01, width)
	width = math_min(sizeTimerWidth, width)
	GVAR.MainFrame.TickTimerTexture:SetWidth(width)

	-- channel timer
	local width = ((channelStart + channelDura - curTime1000) / channelDura) * sizeTimerWidth
	width = math_max(0.01, width)
	width = math_min(sizeTimerWidth, width)
	GVAR.MainFrame.ChannelTimerTexture:SetWidth(width)
end
-- -------------------------------------------------------------------------- --
-- ---------------------------------------------------------------------------------------------------------------------



-- ---------------------------------------------------------------------------------------------------------------------
local function OnEvent(self, event, ...)
	--print(event, ...)
	if event == "UNIT_HEALTH" then
		DrainSouler:CheckHealth()

	elseif event == "PLAYER_DAMAGE_DONE_MODS" then
		DrainSouler:UpdateDetailedInfo()

	elseif event == "PLAYER_TARGET_CHANGED" then
		DrainSouler:CheckTarget()

	elseif event == "COMBAT_LOG_EVENT_UNFILTERED" then
		local _, event, _, sourceGUID, _, _, _, destGUID, _, _, _, _, spellName, _, amount = ... -- timestamp, event, hideCaster, sourceGUID, sourceName, sourceFlags, sourceRaidFlags, destGUID, destName, destFlags, destRaidFlags, spellId, spellName, spellSchool, amount
		if not channelSpells[spellName] then return end
		if sourceGUID ~= PlayerGUID then return end
		if destGUID ~= startDestGUID then return end
		if event ~= "SPELL_PERIODIC_DAMAGE" then return end
		local _, _, _, _, startTime, endTime = UnitChannelInfo("player")
		if not startTime then return end
		if not endTime then return end
		local curTime1000 = GetTime()*1000
		local DrainSouler_Options = DrainSouler_Options
		local spellint = channelSpells[spellName].int
		channelDura = endTime - channelStart
		channelRemain = channelDura - (curTime1000 - channelStart)
		if DrainSouler_Options.DetailInfo then
			DrainSouler:UpdateDetailedInfo()
		end
		channelTick = channelTick + 1
		curTickDura = tickDura
		curTickStart = curTime1000
		--print("DMG--", math_floor(channelRemain), channelDura, endTime, "#-->", channelTick, "D:", math_floor(curTickDura), "S:", curTickStart)
		--print("--DMG--", channelTick, amount)
		--print("--DMG--", channelTick, curTickDura)
		if DrainSouler_Options.TickSound[spellint] > 1 then
			PlaySoundFile(soundsTick[ DrainSouler_Options.TickSound[spellint] ].path, soundChannel[ DrainSouler_Options.TickChannel[spellint] ].channel)
		end
		GVAR.MainFrame.TickText:SetText(channelTick)
		GVAR.MainFrame.DMGText:SetText(amount)

	elseif event == "UNIT_SPELLCAST_SUCCEEDED" then
		local _, spellName = ...
		if not channelSpells[spellName] then return end
		local _, _, _, _, startTime, endTime = UnitChannelInfo("player")
		--print("STD-#", startTime, endTime, GetTime(), (GetTime()*1000), "#", UnitGUID("target"))
		if not startTime then return end
		if not endTime then return end
		--print("STD-#", spellName, endTime - startTime)
		startDestGUID = UnitGUID("target")
		startHaste = UnitSpellHaste("player")
		startDamage = GetSpellBonusDamage(6)
		tickDura = channelSpells[spellName].baseTickDura/((100+startHaste)/100)
		-- HarvestLife and DrainLife works different compared to DrainSoul and MaleficGrasp
		-- HarvestLife and DrainLife always tick 5/6 times, in rare cases 7 times...
		-- needs check if this ever change
		if spellName == HarvestLifeName or
		   spellName == DrainLifeName
		then
			channelDura = (channelSpells[spellName].spellDura/((100+startHaste)/100))
		else
			channelDura = endTime - startTime
		end
		channelStart = startTime
		channelRemain = channelDura
		channelTick = 0
		curTickDura = tickDura
		curTickStart = startTime
		channeling = true
		--print("STD-*", "#", math_floor(tickDura))
		GVAR.MainFrame.TickText:SetText("")
		if DrainSouler_Options.DetailInfo then
			GVAR.MainFrame.DurationText:SetText(string.format("%.2f", channelDura / tickDura).." | "..math_floor(tickDura).."ms | "..math_floor(channelDura).."ms")
			GVAR.MainFrame.ModifiersText:SetText((math_floor(startHaste * mult2 + 0.5) / mult2).." | "..startDamage)
		end
		DrainSouler:RegisterEvent("COMBAT_LOG_EVENT_UNFILTERED")
		GVAR.MainFrame.TickTimer:SetScript("OnUpdate", TickUpdate)
		GVAR.MainFrame.TickTimerTexture:SetTexture(channelSpells[spellName].timerColor[1], channelSpells[spellName].timerColor[2], channelSpells[spellName].timerColor[3], 1)
		GVAR.MainFrame.TickTimer:Show()
		GVAR.MainFrame.ChannelTimerTexture:SetTexture(channelSpells[spellName].timerColor[1], channelSpells[spellName].timerColor[2], channelSpells[spellName].timerColor[3], 1)
		GVAR.MainFrame.ChannelTimer:Show()
		GVAR.MainFrame.SpellIconTexture:SetTexture(channelSpells[spellName].spellIcon)
		GVAR.MainFrame.SpellIcon:Show()

	elseif event == "UNIT_SPELLCAST_CHANNEL_UPDATE" then
		if not channeling then return end
		local _, spellName = ...
		if not channelSpells[spellName] then return end
		local _, _, _, _, startTime, endTime = UnitChannelInfo("player")
		if not startTime then return end
		if not endTime then return end
		local curTime1000 = GetTime()*1000
		channelDura = endTime - channelStart
		channelRemain = channelDura - (curTime1000 - channelStart)
		--print("---", channelTick,"-", tickDura, curTime1000, curTickStart, "ct1000-tstart:", math_floor(dif), "|-->", tickDura+dif)-- - (curTime1000 - startTime))
		curTickDura = tickDura + (curTime1000 - curTickStart)
		curTickStart = curTime1000
		--print("UPD#-", channelStart, GetTime(), GetTime()*1000)
		--print("UPD--", math_floor(channelRemain), channelDura, endTime, "#-->", channelTick, "D:", math_floor(curTickDura), "S:", curTickStart)
		if DrainSouler_Options.DetailInfo then
			DrainSouler:UpdateDetailedInfo()
		end

	elseif event == "UNIT_SPELLCAST_CHANNEL_STOP" then
		local _, spellName = ...
		if not channelSpells[spellName] then return end
		DrainSouler:UnregisterEvent("COMBAT_LOG_EVENT_UNFILTERED")
		channeling = nil
		GVAR.MainFrame.TickTimer:SetScript("OnUpdate", nil)
		GVAR.MainFrame.TickTimer:Hide()
		GVAR.MainFrame.ChannelTimer:Hide()
		GVAR.MainFrame.TickText:SetText("")
		GVAR.MainFrame.DMGText:SetText("")
		GVAR.MainFrame.SpellIcon:Hide()
		if DrainSouler_Options.DetailInfo then
			DrainSouler:UpdateDetailedInfo()
		end

	elseif event == "PLAYER_REGEN_DISABLED" then
		inCombat = true
		DrainSouler:CheckTarget()

	elseif event == "PLAYER_REGEN_ENABLED" then
		inCombat = false
		DrainSouler:CheckTarget()
		-- one cleanup in 180 seconds max.
		local time = GetTime()
		if time < trashTime + 180 then return end
		trashTime = time
		-- remove old (15min/900sec) mob data from mobTable
		time = time - 900
		for guid, timestamp in pairs(mobTable) do
			if timestamp < time then
				mobTable[guid] = nil
			end
		end

	elseif event == "PLAYER_TALENT_UPDATE" then
		DrainSouler:CheckTalent()

	elseif event == "PLAYER_LOGIN" then
		startHaste  = UnitSpellHaste("player") or 0
		startDamage = GetSpellBonusDamage(6) or 0 -- 6 = Shadow
		PlayerGUID  = UnitGUID("player")
		DrainSouler:InitOptions()
		DrainSouler:CreateInterfaceOptions()
		DrainSouler:CreateOptionsFrame()
		DrainSouler:CreateMainFrame()
		DrainSouler:Initialize()
		DrainSouler:UpdateTalentDisplay()
		DrainSouler:UnregisterEvent("PLAYER_LOGIN")
	end
end
-- ---------------------------------------------------------------------------------------------------------------------

DrainSouler:RegisterEvent("PLAYER_TALENT_UPDATE")
DrainSouler:RegisterEvent("PLAYER_LOGIN")
DrainSouler:SetScript("OnEvent", OnEvent)