-------------------------------------------------------------------------------
-- ElvUI Currency LDB Plugin
-- By Lockslap <adam at lockslap dot com>
-------------------------------------------------------------------------------
CurrencyLDB			= LibStub("AceAddon-3.0"):NewAddon("ElvUI_CurrencyLDB", "AceConsole-3.0", "AceEvent-3.0", "AceTimer-3.0")
CurrencyLDB.version	= GetAddOnMetadata("ElvUI_CurrencyLDB", "Version")
local _G			= getfenv(0)
local E, L, V, P, G = unpack(ElvUI); --Inport: Engine, Locales, PrivateDB, ProfileDB, GlobalDB
local DT			= E:GetModule("DataTexts")
local LibQTip		= LibStub("LibQTip-1.0")
local Crayon		= LibStub:GetLibrary("LibCrayon-3.0")
local LDB			= LibStub("LibDataBroker-1.1"):NewDataObject("ElvUI_CurrencyLDB", {
	type	= "data source",
	icon	= "Interface\\MoneyFrame\\UI-GoldIcon",
	label	= "ElvUI_CurrencyLDB",
	text	= "ElvUI_CurrencyLDB"
})

local format = string.format
local split = string.split
local sort = table.sort

local Tip
local anchor
local minimize = "|TInterface\\BUTTONS\\UI-PlusButton-Up:0|t"
local maximize = "|TInterface\\BUTTONS\\UI-MinusButton-Up:0|t"

-- for icons
local goldicon = "\124TInterface\\MoneyFrame\\UI-GoldIcon:%d:%d:1:0\124t"
local silvericon = "\124TInterface\\MoneyFrame\\UI-SilverIcon:%d:%d:1:0\124t"
local coppericon = "\124TInterface\\MoneyFrame\\UI-CopperIcon:%d:%d:1:0\124t"

local entryLine = "|cff%s%s|r"
local colorString = "%02x%02x%02x"

local spent, profit = 0, 0

-- currencies
local currencies = {
	[L["Conquest Points"]]					= 390,
	[L["Honor Points"]]						= 392,
	[L["Tol Barad Commendation"]]			= 391,
	[L["Essence of Corrupted Deathwing"]]	= 615,
	[L["Justice Points"]]					= 395,
	[L["Mote of Darkness"]]					= 614,
	[L["Valor Points"]]						= 396,
	[L["Champion's Seals"]]					= 241,
	[L["Chef's Award"]]						= 402,
	[L["Dalaran Cooking Award"]]			= 81,
	[L["Dalaran Jewelcrafter's Token"]]		= 61,
	[L["Darkmoon Prize Ticket"]]			= 515,
	[L["Illustrious Jewelcrafter's Token"]]	= 361,
	[L["Mark of the World Tree"]]			= 416,
}
local archy = {}

function CurrencyLDB:HasArchaeology()
	local _, _, archy = GetProfessions()
	return archy and true or false
end

-- from ElvUI\modules\datatexts\gold.lua
local function formatMoney(money, color)
	local gold = floor(math.abs(money) / 10000)
	local silver = mod(floor(math.abs(money) / 100), 100)
	local copper = mod(floor(math.abs(money)), 100)
	
	if color == nil then color = "ffffff" end
	
	local goldabbrev = curr_db.useCoins == true and goldicon:format(curr_db.coinSize, curr_db.coinSize) or L.goldabbrev
	local silverabbrev = curr_db.useCoins == true and silvericon:format(curr_db.coinSize, curr_db.coinSize) or L.silverabbrev
	local copperabbrev = curr_db.useCoins == true and coppericon:format(curr_db.coinSize, curr_db.coinSize) or L.copperabbrev
	
	if gold ~= 0 then
		return format("|cff%s%s|r" .. goldabbrev .. " |cff%s%s|r" .. silverabbrev.." |cff%s%s|r" .. copperabbrev, color, gold, color, silver, color, copper)
	elseif silver ~= 0 then
		return format("|cff%s%s|r" .. silverabbrev.." |cff%s%s|r" .. copperabbrev, color, silver, color, copper)
	else
		return format("|cff%s%s|r" .. copperabbrev, color, copper)
	end
end

local function displayBags()
	local free, total, used = 0, 0, 0
	for i = 0, NUM_BAG_SLOTS do
		free, total = free + GetContainerNumFreeSlots(i), total + GetContainerNumSlots(i)
	end
	return ("%s: |cff%s%d/%d|r"):format(L["Bags"], Crayon:GetThresholdHexColor(free / total), free, total)
end

local function OnEvent(self, event, ...)
	if not IsLoggedIn() then return end
	
	local newmoney = GetMoney()
	local oldmoney = curr_gold[UnitName("player")]["gold"] or newmoney
	local change = newmoney - oldmoney
	
	if oldmoney > newmoney then
		spent = spent - change
	else
		profit = profit + change
	end
	
	-- update ldb
	CurrencyLDB:UpdateCharacterGold()
	CurrencyLDB:UpdateLDBText()
end

function CurrencyLDB:UpdateCharacterGold()
	local player, _ = UnitName("player")
	local _, class = UnitClass("player")
	local rgb = RAID_CLASS_COLORS[class]
	local hex = ("%02x%02x%02x"):format(rgb.r * 255, rgb.g * 255, rgb.b * 255)
	if not curr_gold then curr_gold = {} end
	curr_gold[player] = {
		gold = GetMoney(),
		colored = ("|cff%s%s|r"):format(hex, player)
	}
end

function CurrencyLDB:UpdateLDBText()
	LDB.text = curr_db.ldbText == "money" and formatMoney(GetMoney()) or displayBags()
end

local function ToggleGuildFrame()
	if IsInGuild() then
		if not GuildFrame then LoadAddOn("Blizzard_GuildUI") end
		GuildFrame_Toggle()
		GuildFrame_TabClicked(GuildFrameTab2)
	else
		if not LookingForGuildFrame then LoadAddOn("Blizzard_LookingForGuildUI") end
		if LookingForGuildFrame then LookingForGuildFrame_Toggle() end
	end
end

local function doAction(action)
	if action == "allbags" then
		ToggleAllBags()
	elseif action == "currency" then
		ToggleCharacter("TokenFrame")
	elseif action:find("bag([0-9]{1})") then
		local bag = action:match("bag([0-9]{1})")
		ToggleBag(bag)
	elseif action == "character" then
		ToggleCharacter("PaperDollFrame")
	elseif action == "pet" then
		ToggleCharacter("PetPaperDollFrame")
	elseif action == "reputation" then
		ToggleCharacter("ReputationFrame")
	elseif action == "friends" then
		ToggleFriendsFrame(1)
	elseif action == "guild" then
		ToggleGuildFrame()
	elseif action == "garbage" then
		collectgarbage("collect")
	end
end

function LDB:OnClick(button)
	if button == "LeftButton" then
		doAction(curr_db.dtLeft)
	elseif button == "RightButton" then
		doAction(curr_db.dtRight)
	end
end

local function doEntryAction(frame, action, id, watched, unused)
	if action == "watched" then
		SetCurrencyBackpack(id, watched)
	elseif action == "unused" then
		SetCurrencyUnused(id, unused)
	elseif action == "link" then
		CurrencyLDB:Print(GetCurrencyLink(id))
	end
	LDB.OnEnter(frame)
end

local function EntryClick(frame, info, button)
	local name, isUnused, isWatched = split(":", info)
	if not currencies[name] then return end
	isUnused = isUnused == "true" and 0 or 1
	isWatched = isWatched == "true" and 0 or 1
	if button == "LeftButton" then
		doEntryAction(frame, curr_db.entryLeft, currencies[name], isWatched, isUnused)
	elseif button == "RightButton" then
		doEntryAction(frame, curr_db.entryRight, currencies[name], isWatched, isUnused)
	end
end

local function ArchyEntryClick(frame, info, button)
	CurrencyLDB:Print(GetCurrencyLink(info))
end

local function getHeaderName(header)
	return header:lower():gsub(" ", ""):gsub("%.", "")
end

local function HeaderClick(frame, section)
	if not IsAltKeyDown() then
		curr_db.toggles[section] = not curr_db.toggles[section]
	else
		-- alt click hides section completely
		if section == "archaeologyfragments" then
			curr_db.archy = not curr_db.archy
		else
			curr_db[section] = not curr_db[section]
		end
	end
	LDB.OnEnter(frame)
end

local function UpdateFragments()
	archy = {}
	for index, value in pairs(curr_db.fragments) do
		if value == true then
			archy[#archy + 1] = index:match("frag_(.+)")
		end
	end
end

function LDB.OnEnter(self)
	if curr_db.combat == true and InCombatLockdown() then return end
	local lastHeader = ""
	CurrencyLDB:UpdateCharacterGold()
	
	if LibQTip:IsAcquired("CurrencyLDBTip") then
		Tip:Clear()
	else
		Tip = LibQTip:Acquire("CurrencyLDBTip", 6, "LEFT", "LEFT", "CENTER", "RIGHT", "RIGHT", "CENTER")
		Tip:SmartAnchorTo(self:GetParent())
		Tip:SetAutoHideDelay(0.1, self)
		Tip:SetScript("OnShow", function(ttskinself) ttskinself:SetTemplate(curr_db.tooltipSkin) end)
	end
	
	-- add gold information for all characters
	if curr_db.availablefunds and curr_gold then
		line = Tip:AddHeader()
		if curr_db.toggles[getHeaderName(L["Available Funds"])] then
			Tip:SetCell(line, 1, maximize .. " " .. L["Available Funds"], "LEFT", 4)
		else
			Tip:SetCell(line, 1, minimize .. " " .. L["Available Funds"], "LEFT", 4)
		end
		Tip:SetLineScript(line, "OnMouseUp", HeaderClick, getHeaderName(L["Available Funds"]))
		Tip:AddSeparator()
		if curr_db.toggles[getHeaderName(L["Available Funds"])] then
			local totalGold = 0
			for player, data in pairs(curr_gold) do
				line = Tip:AddLine()
				Tip:SetCell(line, 1, data.colored, "LEFT", 2)
				Tip:SetCell(line, 4, formatMoney(data.gold), "RIGHT", 2)
				totalGold = totalGold + data.gold
			end
			Tip:AddSeparator()
			line = Tip:AddLine()
			Tip:SetCell(line, 1, entryLine:format("ffff00", L["Total Funds:"]), "LEFT", 2)
			Tip:SetCell(line, 4, formatMoney(totalGold), "RIGHT", 2)
		end
	end
	
	if curr_db.session then
		if curr_db.availablefunds then Tip:AddLine(" ") end
		line = Tip:AddHeader()
		if curr_db.toggles[getHeaderName(L["Session"])] then
			Tip:SetCell(line, 1, maximize .. " " .. L["Session"], "LEFT", 4)
		else
			Tip:SetCell(line, 1, minimize .. " " .. L["Session"], "LEFT", 4)
		end
		Tip:SetLineScript(line, "OnMouseUp", HeaderClick, getHeaderName(L["Session"]))
		Tip:AddSeparator()
		if curr_db.toggles[getHeaderName(L["Session"])] then
			line = Tip:AddLine()
			Tip:SetCell(line, 1, L["Earned"], "LEFT", 2)
			Tip:SetCell(line, 4, formatMoney(profit), "RIGHT", 2)
			line = Tip:AddLine()
			Tip:SetCell(line, 1, L["Spent"], "LEFT", 2)
			Tip:SetCell(line, 4, formatMoney(spent), "RIGHT", 2)
			
			if profit < spent then
				line = Tip:AddLine()
				Tip:SetCell(line, 1, L["Deficit"], "LEFT", 2)
				Tip:SetCell(line, 4, formatMoney(profit - spent, "ff0000"), "RIGHT", 2)
				line = Tip:AddLine()
			elseif (profit - spent) > 0 then
				line = Tip:AddLine()
				Tip:SetCell(line, 1, L["Profit"], "LEFT", 2)
				Tip:SetCell(line, 4, formatMoney(profit - spent, "00ff00"), "RIGHT", 2)
			end
		end
	end
	
	if curr_db.dungeonandraid and curr_db.availablefunds then Tip:AddLine(" ") end
	
	for i = 1, GetCurrencyListSize() do
		local name, isHeader, isExpanded, isUnused, isWatched, count, icon = GetCurrencyListInfo(i)
		if isHeader then
			if curr_db.hideUnused and name ~= "Unused" then
				local header = getHeaderName(name)
				if curr_db[header] then
					if i > 1 and curr_db[lastHeader] then
						Tip:AddLine(" ")
					end
					line = Tip:AddHeader()
					if curr_db.toggles[header] then
						Tip:SetCell(line, 1, maximize .. " " .. name, "LEFT", 3)
					else
						Tip:SetCell(line, 1, minimize .. " " .. name, "LEFT", 3)
					end
					Tip:SetLineScript(line, "OnMouseUp", HeaderClick, header)
					Tip:AddSeparator()
					lastHeader = header
				end
			end
		else
			if curr_db.toggles[lastHeader] and curr_db[lastHeader] then
				if not isUnused or (isUnused and not curr_db.hideUnused) then
					if not curr_db.hideZero then
						line = Tip:AddLine()
						Tip:SetCell(line, 1, "|T" .. icon .. ":0|t", "CENTER")
						if not curr_db.colorZero or (curr_db.colorZero and count > 0) then
							Tip:SetCell(line, 2, name, "LEFT")
							Tip:SetCell(line, 4, count, "RIGHT", 2)
						elseif curr_db.colorZero and count == 0 then
							Tip:SetCell(line, 2, entryLine:format(colorString:format(curr_db.zeroColor.r * 255, curr_db.zeroColor.g * 255, curr_db.zeroColor.b * 255), name), "LEFT")
							Tip:SetCell(line, 4, entryLine:format(colorString:format(curr_db.zeroColor.r * 255, curr_db.zeroColor.g * 255, curr_db.zeroColor.b * 255), count), "RIGHT", 2)
						end
						Tip:SetLineScript(line, "OnMouseUp", EntryClick, name .. ':' .. tostring(isUnused) .. ":" .. tostring(isWatched))
					elseif curr_db.hideZero and count > 0 then
						line = Tip:AddLine()
						Tip:SetCell(line, 1, "|T" .. icon .. ":0|t", "CENTER")
						Tip:SetCell(line, 2, name, "LEFT")
						Tip:SetCell(line, 4, count, "RIGHT", 2)
						Tip:SetLineScript(line, "OnMouseUp", EntryClick, name .. ':' .. tostring(isUnused) .. ":" .. tostring(isWatched))
					end
				end
			end
		end
	end
	
	if CurrencyLDB:HasArchaeology() and curr_db.archy then
		local fragments = {}
		UpdateFragments()
		Tip:AddLine(" ")
		line = Tip:AddHeader()
		if curr_db.toggles[getHeaderName(L["Archaeology Fragments"])] then
			Tip:SetCell(line, 1, maximize .. " " .. L["Archaeology Fragments"], "LEFT", 4)
		else
			Tip:SetCell(line, 1, minimize .. " " .. L["Archaeology Fragments"], "LEFT", 4)
		end
		Tip:SetLineScript(line, "OnMouseUp", HeaderClick, getHeaderName(L["Archaeology Fragments"]))
		Tip:AddSeparator()
		if curr_db.toggles[getHeaderName(L["Archaeology Fragments"])] then
			for i = 1, #archy do
				local name, count, icon, _, _, _, _ = GetCurrencyInfo(archy[i])
				fragments[#fragments + 1] = {
					name	= name,
					id		= archy[i],
					count	= count,
					icon	= icon
				}
			end
			
			if #fragments > 0 then
				sort(fragments, function(a, b) return a['name'] < b['name'] end)
				for i = 1, #fragments do
					if not curr_db.hideZero then
						line = Tip:AddLine()
						Tip:SetCell(line, 1, "|TInterface\\Icons\\" .. fragments[i].icon .. ":12:12:1:0|t", "CENTER")
						if not curr_db.colorZero or (curr_db.colorZero and fragments[i].count > 0) then
							Tip:SetCell(line, 2, fragments[i].name, "LEFT", 2)
							Tip:SetCell(line, 4, fragments[i].count, "RIGHT", 2)
						elseif curr_db.colorZero and fragments[i].count == 0 then
							Tip:SetCell(line, 2, entryLine:format(colorString:format(curr_db.zeroColor.r * 255, curr_db.zeroColor.g * 255, curr_db.zeroColor.b * 255), fragments[i].name), "LEFT", 2)
							Tip:SetCell(line, 4, entryLine:format(colorString:format(curr_db.zeroColor.r * 255, curr_db.zeroColor.g * 255, curr_db.zeroColor.b * 255), fragments[i].count), "RIGHT", 2)
						end
						Tip:SetLineScript(line, "OnMouseUp", ArchyEntryClick, fragments[i].id)
					elseif curr_db.hideZero and fragments[i].count > 0 then
						line = Tip:AddLine()
						Tip:SetCell(line, 1, "|TInterface\\Icons\\" .. fragments[i].icon .. ":12:12:1:0|t", "CENTER")
						Tip:SetCell(line, 2, fragments[i].name, "LEFT", 2)
						Tip:SetCell(line, 4, fragments[i].count, "RIGHT", 2)
						Tip:SetLineScript(line, "OnMouseUp", ArchyEntryClick, fragments[i].id)				
					end
				end
			end
		end
	end
	
	Tip:UpdateScrolling()
	Tip:Show()
end

function CurrencyLDB:OnInitialize()
	-- hack so you dont have to reset settings after every update
	for index, value in pairs(self.defaults) do
		if type(curr_db[index]) == "nil" then
			curr_db[index] = value
		end
	end
	
	if not curr_db.fragments then
		curr_db.fragments = {
			['frag_398']	= true,	-- Draenei
			['frag_384']	= true,	-- Dwarf
			['frag_393']	= true,	-- Fossil
			['frag_400']	= true,	-- Nerubian
			['frag_394']	= true,	-- Night Elf
			['frag_397']	= true,	-- Orc
			['frag_401']	= true,	-- Tol'vir
			['frag_385']	= true,	-- Troll
			['frag_399']	= true,	-- Vrykul
		}
	end
	
	if not curr_db.toggles then
		curr_db.toggles = {
			['availablefunds']			= true,
			['session']					= true,
			['dungeonandraid']			= true,
			['miscellaneous']			= true,
			['playervsplayer']			= true,
			['unused']					= true,
			['archaeologyfragments']	= true,
		}
	end
	
	for index, value in pairs(self.defaults.fragments) do
		if curr_db.fragments[index] == "" then
			curr_db.fragments[index] = value
		end
	end
	
	for index, value in pairs(self.defaults.toggles) do
		if curr_db.toggles[index] == "" then
			curr_db.toggles[index] = value
		end
	end
end


function CurrencyLDB:CHAT_MSG_ADDON(self, event, ...)
	if not curr_db.checkVersion then return end
	local addon, msg, _ = ...
	if addon == "ElvUICurrencyLDB" then
		local name, version = msg:match("(%a+):"), msg:match(":(.*)")
		if tonumber(version) ~= nil and tonumber(self.newVersion) ~= nil and tonumber(version) > tonumber(self.newVersion) then
			self.newVersion = version
			self:Print(format(L["New version |cff1b628c(%s)|r available."]))
		end
	end
end

function CurrencyLDB:SendVersionData(arg)
	if not curr_db.checkVersion then return end
	-- figure out which channel to use
	if IsInGuild() then
		SendAddonMessage("ElvUICurrencyLDB", arg, "GUILD")
	end
end

function CurrencyLDB:OnEnable()
	-- send addon message for newer version
	self:ScheduleTimer("SendVersionData", 2, "VERSION:" .. CurrencyLDB.version)
	self.newVersion = self.version
end

CurrencyLDB:RegisterEvent("PLAYER_ENTERING_WORLD", OnEvent)
CurrencyLDB:RegisterEvent("SEND_MAIL_MONEY_CHANGED", OnEvent)
CurrencyLDB:RegisterEvent("SEND_MAIL_COD_CHANGED", OnEvent)
CurrencyLDB:RegisterEvent("PLAYER_TRADE_MONEY", OnEvent)
CurrencyLDB:RegisterEvent("TRADE_MONEY_CHANGED", OnEvent)
CurrencyLDB:RegisterEvent("PLAYER_MONEY", OnEvent)
CurrencyLDB:RegisterEvent("BAG_UPDATE", displayBags)
CurrencyLDB:RegisterEvent("CHAT_MSG_ADDON")