-------------------------------------------------------------------------------
-- ElvUI Currency LDB Plugin
-- By Lockslap <adam at lockslap dot com>
-------------------------------------------------------------------------------
local E, L, V, P, G = unpack(ElvUI); --Inport: Engine, Locales, PrivateDB, ProfileDB, GlobalDB
--if not CurrencyLDB then return end

CurrencyLDB.defaults = {
	['debug']						= false,
	['combat']						= true,
	['checkVersion']				= true,
	['useCoins']					= true,
	['coinSize']					= 12,
	['session']						= true,
	['hideZero']					= false,
	['colorZero']					= true,
	['zeroColor']					= {r = 0.6, g = 0.6, b = 0.6},
	['tooltipSkin']					= "Transparent",
	['hideUnused']					= true,
	
	-- ldb options
	['ldbText']						= "money",
	['dtLeft']						= "allbags",
	['dtRight']						= "currency",
	['entryLeft']					= "link",
	['entryRight']					= "unused",
	
	-- sections
	['availablefunds']				= true,
	['cataclysm']					= true,
	['dungeonandraid']				= true,
	['miscellaneous']				= true,
	['playervsplayer']				= true,
	['archy']						= true,
	
	-- archaeology options
	['fragments']					= {
		['frag_398']	= true,	-- Draenei
		['frag_384']	= true,	-- Dwarf
		['frag_393']	= true,	-- Fossil
		['frag_400']	= true,	-- Nerubian
		['frag_394']	= true,	-- Night Elf
		['frag_397']	= true,	-- Orc
		['frag_401']	= true,	-- Tol'vir
		['frag_385']	= true,	-- Troll
		['frag_399']	= true,	-- Vrykul
	},
	
	['toggles']		= {
		['availablefunds']			= true,
		['session']					= true,
		['cataclysm']				= true,
		['dungeonandraid']			= true,
		['miscellaneous']			= true,
		['playervsplayer']			= true,
		['unused']					= true,
		['archaeologyfragments']	= true,
	}
}

curr_gold = curr_gold or {}
curr_db = curr_db or CurrencyLDB.defaults

E.Options.args.currldb = {
	order = 9000,
	type = "group",
	name = L["ElvUI_CurrencyLDB"],
	get = function(info) return curr_db[info[#info]] end,
	set = function(info, value) curr_db[info[#info]] = value end,
	args = {
		header = {
			order = 1,
			type = "header",
			name = L["ElvUI_CurrencyLDB v|cff33ffff%s|r by Lockslap (US - Bleeding Hollow)"]:format(CurrencyLDB.version),
		},
		description = {
			order = 2,
			type = "description",
			name = L["ElvUI_CurrencyLDB provides an extended currency datatext using LibDataBroker and LibQTip!\nBased on ElvUI/Tukui - Improved Currency Datatext by Bullzkill."],
		},
		general = {
			order = 3,
			type = "group",
			name = L["General"],
			guiInline = true,
			args = {
				combat = {
					type = "toggle",
					order = 1,
					name = L["Disable In Combat"],
					desc = L["Do not display the tooltip when mousedover in combat."]
				},
				resetsettings = {
					type = "execute",
					order = 2,
					name = L["Reset Settings"],
					desc = L["Reset the settings of this addon to their defaults."],
					func = function()
						curr_db = CurrencyLDB.defaults
						CurrencyLDB:UpdateLDBText()
						CurrencyLDB:Print("Settings were reset to defaults.")
					end,
				},
				resetgold = {
					type = "execute",
					order = 3,
					name = L["Reset Gold Totals"],
					desc = L["Reset the character-based gold totals."],
					func = function()
						curr_gold = {}
						CurrencyLDB:Print("Character Gold Totals Reset!")
					end,
				},
				checkVersion = {
					type = "toggle",
					order = 4,
					name = L["Version Check"],
					desc = L["Check for a new version when logging in."]
				}
			},
		},
		ldbOptions = {
			order = 4,
			type = "group",
			name = L["LDB Options"],
			guiInline = true,
			args = {
				ldbText = {
					type = "select",
					order = 1,
					name = L["LDB Display Mode"],
					desc = L["What do you want displayed in the datatext."],
					values = {
						["money"] = L["Money"],
						["bags"] = L["Bags"]
					},
					set = function(info, value)
						curr_db[info[#info]] = value
						CurrencyLDB:UpdateLDBText()
					end,
				},
				dtLeft = {
					type = "select",
					order = 2,
					name = L["Datatext Left Click"],
					desc = L["Action when the datatext is left clicked."],
					values = {
						["character"]	= L["Toggle Character Frame"],
						["pet"]			= L["Toggle Pet Frame"],
						["reputation"]	= L["Toggle Reputation Frame"],
						["currency"]	= L["Toggle Currency Frame"],
						["allbags"]		= L["Toggle All Bags"],
						["guild"]		= L["Toggle Guild Frame"],
						["friends"]		= L["Toggle Friends Frame"],
						["bag1"]		= L["Toggle Bag 1"],
						["bag2"]		= L["Toggle Bag 2"],
						["bag3"]		= L["Toggle Bag 3"],
						["bag4"]		= L["Toggle Bag 4"],
						["garbage"]		= L["Garbage Collect"],
					}
				},
				dtRight = {
					type = "select",
					order = 3,
					name = L["Datatext Right Click"],
					desc = L["Action when the data text is right clicked."],
					values = {
						["character"]	= L["Toggle Character Frame"],
						["pet"]			= L["Toggle Pet Frame"],
						["reputation"]	= L["Toggle Reputation Frame"],
						["currency"]	= L["Toggle Currency Frame"],
						["allbags"]		= L["Toggle All Bags"],
						["guild"]		= L["Toggle Guild Frame"],
						["friends"]		= L["Toggle Friends Frame"],
						["bag1"]		= L["Toggle Bag 1"],
						["bag2"]		= L["Toggle Bag 2"],
						["bag3"]		= L["Toggle Bag 3"],
						["bag4"]		= L["Toggle Bag 4"],
						["garbage"]		= L["Garbage Collect"],
					}
				},
				entryLeft = {
					type = "select",
					order = 4,
					name = L["Entry Left Click"],
					desc = L["Action when an entry is left clicked."],
					values = {
						["unused"]		= L["Toggle Unused"],
						["watched"]		= L["Toggle Watched"],
						["link"]		= L["Create Link"],
					},
				},
				entryRight = {
					type = "select",
					order = 4,
					name = L["Entry Right Click"],
					desc = L["Action when an entry is right clicked."],
					values = {
						["unused"]		= L["Toggle Unused"],
						["watched"]		= L["Toggle Watched"],
						["link"]		= L["Create Link"],
					},
				},
			}
		},
		sections = {
			order = 5,
			type = "group",
			name = L["Sections"],
			guiInline = true,
			args = {
				availablefunds = {
					type = "toggle",
					order = 1,
					name = L["Available Funds"],
					desc = L["Show/hide the Available Funds section."],
				},
				cataclysm = {
					type = "toggle",
					order = 2,
					name = L["Cataclysm"],
					desc = L["Show/hide the Cataclysm section."],
				},
				dungeonandraid = {
					type = "toggle",
					order = 3,
					name = L["Dungeon and Raid"],
					desc = L["Show/hide the Dungeon and Raid section."],
				},
				miscellaneous = {
					type = "toggle",
					order = 4,
					name = L["Miscellaneous"],
					desc = L["Show/hide the Miscellaneous section."],
				},
				playervsplayer = {
					type = "toggle",
					order = 5,
					name = L["Player vs. Player"],
					desc = L["Show/hide the Player vs. Player section."],
				},
				archy = {
					type = "toggle",
					order = 6,
					disabled = function() return not CurrencyLDB:HasArchaeology() end,
					name = L["Archaeology"],
					desc = L["Show/hide the Archaeology section."]
				},
			}
		},
		display = {
			order = 6,
			type = "group",
			name = L["Display Options"],
			guiInline = true,
			args = {
				tooltipSkin = {
					type = "select",
					order = 2,
					name = L["Tooltip Skin"],
					desc = L["Choose a skin to use for the tooltip."],
					values = {
						["Transparent"] = L["Transparent"],
						["Default"] = L["Default"]
					},
				},
				hideUnused = {
					type = "toggle",
					order = 3,
					name = L["Hide Unused"],
					desc = L["Hide currencies marked as unused."]
				},
				hideZero = {
					type = "toggle",
					order = 4,
					name = L["Hide Zero Entries"],
					desc = L["Hide currencies who's quantity is zero (0)."]
				},
				colorZero = {
					type = "toggle",
					order = 5,
					name = L["Color Zero Entries"],
					desc = L["Color currencies who's quantity is zero (0) differently."],
					set = function(info, value)
						curr_db[info[#info]] = value
						if value == true then curr_db.hideZero = false end
					end,
				},
				zeroColor = {
					type = "color",
					order = 6,
					name = L["Zero Quantity Color"],
					desc = L["Color of currencies who's quantity is zero (0)."],
					get = function(info)
						local t = curr_db[info[#info]]
						return t.r, t.g, t.b
					end,
					set = function(info, r, g, b)
						curr_db[info[#info]].r = r
						curr_db[info[#info]].g = g
						curr_db[info[#info]].b = b
					end,
				},
			}
		},
		gold = {
			order = 7,
			type = "group",
			name = L["Gold Format"],
			guiInline = true,
			args = {
				useCoins = {
					type = "toggle",
					order = 1,
					name = L["Use Coins"],
					desc = L["Use coin icons instead of the ElvUI default abbreviations."],
					set = function(info, value)
						curr_db[info[#info]] = value
						CurrencyLDB:UpdateLDBText()
					end,
				},
				coinSize = {
					type = "range",
					order = 2,
					name = L["Coin Size"],
					desc = L["Size of the coin icons."],
					min = 1, max = 32, step = 1, bigStep = 1,
					disabled = function() return not curr_db.useCoins end,
					set = function(info, value)
						curr_db[info[#info]] = value
						CurrencyLDB:UpdateLDBText()
					end,
				},
				session = {
					type = "toggle",
					order = 3,
					name = L["Display Session Data"],
					desc = L["Add session data to the tooltip."],
				},
			}
		},
		archy = {
			order = 8,
			type = "group",
			name = L["Archaeology Fragments"],
			guiInline = true,
			disabled = function() return not CurrencyLDB:HasArchaeology() end,
			get = function(info) return curr_db.fragments[info[#info]] end,
			set = function(info, value) curr_db.fragments[info[#info]] = value end,
			args = {
				frag_398 = {	-- Draenei
					type = "toggle",
					order = 1,
					disabled = function() return not curr_db.archy end,
					name = L["Draenei"],
					desc = L["Show Draenei fragments."],
				},
				frag_384 = {	-- Dwarf
					type = "toggle",
					order = 2,
					disabled = function() return not curr_db.archy end,
					name = L["Dwarf"],
					desc = L["Show Dwarf fragments."],
				},
				frag_393 = {	-- Fossil
					type = "toggle",
					order = 3,
					disabled = function() return not curr_db.archy end,
					name = L["Fossil"],
					desc = L["Show Fossil fragments."],
				},
				frag_400 = {	-- Nerubian
					type = "toggle",
					order = 4,
					disabled = function() return not curr_db.archy end,
					name = L["Nerubian"],
					desc = L["Show Nerubian fragments."],
				},
				frag_394 = {	-- Night Elf
					type = "toggle",
					order = 5,
					disabled = function() return not curr_db.archy end,
					name = L["Night Elf"],
					desc = L["Show Night Elf fragments."],
				},
				frag_397 = {	-- Orc
					type = "toggle",
					order = 6,
					disabled = function() return not curr_db.archy end,
					name = L["Orc"],
					desc = L["Show Orc fragments."],
				},
				frag_401 = {	-- Tol'vir
					type = "toggle",
					order = 7,
					disabled = function() return not curr_db.archy end,
					name = L["Tol'vir"],
					desc = L["Show Tol'vir fragments."],
				},
				frag_385 = {	-- Troll
					type = "toggle",
					order = 8,
					disabled = function() return not curr_db.archy end,
					name = L["Troll"],
					desc = L["Show Troll fragments."],
				},
				frag_399 = {	-- Vrykul
					type = "toggle",
					order = 9,
					disabled = function() return not curr_db.archy end,
					name = L["Vrykul"],
					desc = L["Show Vrykul fragments."]
				},
			}
		},
	}
}